#include "toolbox.h"
#include <ansi_c.h>
#include <utility.h>
#include <rs232.h>
#include <userint.h>
#include "globales.h"
#include "cred.h" 
#include "UIR_cred.h"

#include "plugin.h"

PInstrument cred_4v =
{
	NULL, // Rempli par 'nouveau'
	
	NULL, // rempli aprs nouveau (par liste.c)
	NULL,
	NULL,
	NULL,
	NULL,
	
	cred_nouveau,
	cred_liberer,
	cred_configure,
	cred_getconfig,
	cred_get_nom,
	cred_set_nom,
	CRED_arreter_comint,
	NULL,
	CRED_initialisation,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	
	cred_nb_voie,
	cred_get_nom_voie,
	cred_mesure,
	cred_numero,
	
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

/* cred.c
Contient les fonctions spcifiques au fonctionnement de la carte du CRED
*/

union voie_n		voie_2,voie_3;//contiennent les parametres de regulation pour PWM
char 				pwm,T_echantillon=1;
int sem=0; //pseudo smaphore

// Fonction d'initialisation de la communication avec la carte du CRED connecte sur [port]
// Sans la rgulation de temprature

int CRED_initialisation(PInstrument *cred)
{
	infos_cred *data_cred = cred->structure;
	int err=0,i=0;
	char instruction[1];

	//initialisation des parametres de PWM pour le cas ou on ne regule pas sur l'une des voies	
	for(i=0;i<NBR_BYTE_VOIE;i++)	
		{
		voie_2.bytes[i]=0;
		voie_3.bytes[i]=0;
		}

	// Initialisation Zone Mmoire.
	data_cred->zone_memoire[0]=NotANumber();
	data_cred->zone_memoire[1]=NotANumber();
	data_cred->zone_memoire[2]=NotANumber();
	data_cred->zone_memoire[3]=NotANumber();


	//Initialisation Numro Mesure.
	data_cred->numero_mesure[0] = 0;
	data_cred->numero_mesure[1] = 0;  
	data_cred->numero_mesure[2] = 0;  
	data_cred->numero_mesure[3] = 0;  

	//Attente d'autorisation d'utilisation du pc par le HC11 
	err = OpenComConfig (data_cred->port_com, "COM1", 9600, 0, 8, 1, 512, 512); // Ouverture du port COM1

	if (err<0)
		{
		MessagePopup ("Attention", "Impossible d'ouvrir le port demand");
		return err;
		}
	SetComTime (data_cred->port_com, 10.0); 

	// Synchronisation de l'oprateur
	MessagePopup("Attention","Veuillez :\n Rinitialiser la carte du CRED\n Activer l'utilisation du PC");

	do
		{
		CRED_RS232_en_attente(data_cred);
		ComRd (data_cred->port_com, instruction, 1);
		}while (instruction[0]!=VALID_PC);

	CRED_requete(data_cred);	// On interroge le HC11 pour savoir si il est pret a recevoir des infos,
			    
	//on envoie:
	//1 -  pwm(0 ou 1)
	//2  - les parametres PWM voie1 puis les parametres PWM voie2 			   
	ComWrtByte (data_cred->port_com, pwm);
	ComWrt (data_cred->port_com, voie_2.bytes, NBR_BYTE_VOIE);
	ComWrt (data_cred->port_com, voie_3.bytes, NBR_BYTE_VOIE);

	//Attente d'autorisation par HC11 d'envoi de TE
	do
	{
	CRED_RS232_en_attente(data_cred);
	ComRd (data_cred->port_com, instruction, 1);
	}while (instruction[0]!=VALID_TE);

	CRED_requete(data_cred);	// On interroge le HC11 pour savoir si il est pret a recevoir des infos,
	err = ComWrtByte (data_cred->port_com, data_cred->temps_echantillonnage); // Envoi de la frquence d'chantillonage

	ComWrtByte (data_cred->port_com, ACK);
	
	sem=0;
	SetComTime (data_cred->port_com, (double) data_cred->temps_echantillonnage); //Rduction du timeout  la frquence d'chantillonage pour tre prt  lire la donne suivante directement
	
	// Installation d'une interruption  la rception de ETX
	err = InstallComCallback (data_cred->port_com, 0x0002 ,0,ETX, CRED_comint, (void *)data_cred);
	return err;
}

// Fonction d'interruption sur le port
void CVICALLBACK CRED_comint(int COMPort, int eventMask, void *CallbackData)
{
	infos_cred *data_cred;

	int err,nboctets_file,numoctet=0;
	struct data donnees;
	union data_mesure mesures; 	  //contient les mesures de frquence et d'epsilon

	data_cred = (infos_cred *) CallbackData; 

	if (!(eventMask & 0x0002))
		return;

	if (data_cred==NULL) //Au cas o l'arrt de l'acquisition libre le pointeur et o il y a encore un interruption
		return ;

	// Rcuprer le nombre d'octets dans la file
	nboctets_file = GetInQLen (data_cred->port_com);
	if (nboctets_file ==0) // l 'interruption qui ne sert  rien.
		Delay(0.1);
	nboctets_file = GetInQLen (data_cred->port_com);
	if (nboctets_file ==0) // l 'interruption qui ne sert VRAIMENT  rien.
		return;			// on sort
	
	// Vider le buffer pour eviter de lire les ETX
	err=ComRdByte (data_cred->port_com);
	if (err==0)
	{
		// On a lu 0 alors qu'on devait lire ETX
		ComRdByte (data_cred->port_com);
	}
	else
	{
		err++; // Servait  mettre un breakpoint
	}
	
	// Lancement de l'envoi
	err = ComWrtByte (data_cred->port_com, ACK);
	do
	{
		mesures.bytes[numoctet] = ComRdByte (data_cred->port_com);
		err = ComWrtByte (data_cred->port_com, ACK); // Accus de rception
		numoctet++;
	}while(numoctet<NBR_BYTE_DONNEES);
	
	CRED_inversion_entier(mesures.bytes);
	CRED_inversion_short_mesures(mesures.bytes);
	donnees=mesures.valeur;

	// Rcupration des rsultats de mesure
	data_cred->zone_memoire[0] = (double) donnees.freq_1;
	data_cred->zone_memoire[1] = (double) donnees.freq_2;
	data_cred->zone_memoire[2] = (double) donnees.freq_3;
	data_cred->zone_memoire[3] = (double) donnees.freq_4;

	// Incrmentation du nombre de mesures.
	data_cred->numero_mesure[0]++;   
	data_cred->numero_mesure[1]++; 
	data_cred->numero_mesure[2]++; 
	data_cred->numero_mesure[3]++; 
}

// Fonction d'arrt de gestion de l'interruption sur le com
static void CRED_arreter_comint(PInstrument *cred)
{
	infos_cred *data_cred = cred->structure;
	int err;
	err = InstallComCallback (data_cred->port_com, 0, 0, 0, CRED_comint, NULL);

	//return err;
}

/////////////////////////////////////////////////////////////////////////////////
// Fonction inversant les octets de poids fort et faible pour format MOTOROLA
/////////////////////////////////////////////////////////////////////////////////

// utilis pour les parametres PWM
void CRED_inversion_short_pwm(char* tableau)
{
	char tmp,i;
	for(i=2;i<=7;i=i+2)
	{
		tmp=tableau[i];
		tableau[i]=tableau[i+1];
		tableau[i+1]=tmp;
	}
}

void CRED_inversion_short_mesures(char* tableau)
{
	char tmp,i;
	for(i=0;i<=3;i=i+2)
	{
		tmp=tableau[i];
		tableau[i]=tableau[i+1];
		tableau[i+1]=tmp;
	}
}
void CRED_inversion_entier(char* tab)
{
	char i,j,tmp[4];
	for(i=4;i<=19;i=i+4)   // D'o sort le 19 ?
	{
	  	tmp[0]=tab[i+3];
	  	tmp[1]=tab[i+2];
		tmp[2]=tab[i+1];
		tmp[3]=tab[i];
		for(j=0;j<=3;j++)
		{tab[j+i]=tmp[j];}
	}
}

///////////////////////////////////		
//Fonctions concernant la RS232		
///////////////////////////////////

//procedure requete acquittement
void CRED_requete(infos_cred *data_cred)
{
int status_rs232_hc11=0,err; 

err = ComWrtByte (data_cred->port_com, ETX);
do
	{
	status_rs232_hc11 = ComRdByte (data_cred->port_com);
	if (status_rs232_hc11<0)
		{
		err = ComWrtByte (data_cred->port_com, ACK); // pour peut-tre dbloquer la carte ??
		}
	}
while(status_rs232_hc11!=ACK);
}

//procedure d'attente d'info en provenance du  HC11
void CRED_RS232_en_attente(infos_cred *data_cred)
{
int status_rs232_hc11=0,err;

	do
	{
		status_rs232_hc11 = ComRdByte (data_cred->port_com);
		if (status_rs232_hc11<0)
		{
		err = ComWrtByte (data_cred->port_com, ACK); // pour peut-tre dbloquer la carte ??
		}

	}
	while (status_rs232_hc11!=ETX);
	err = ComWrtByte (data_cred->port_com, ACK);
	return;

}						

/////////////////
// Gestion CVI //
/////////////////

static void cred_nouveau(PInstrument **cred)
{
	*cred = (PInstrument *)malloc(sizeof(PInstrument));
	memcpy (*cred, &cred_4v, sizeof(PInstrument));
	(*cred)->structure = (infos_cred *)malloc(sizeof(infos_cred));
	((infos_cred *)(*cred)->structure)->port_com = 1;
	((infos_cred *)(*cred)->structure)->temps_echantillonnage = 1;
	sprintf(((infos_cred *)(*cred)->structure)->nom_instru, "Carte CRED");
}

static void cred_liberer(PInstrument *instru)
{
	free(instru->structure);
	free(instru);
}

static void cred_get_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(nom_instru, ((infos_cred *)instrument->structure)->nom_instru, TAILLE_NOM_INSTRU-1);
}

static void cred_set_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(((infos_cred *)instrument->structure)->nom_instru, nom_instru, TAILLE_NOM_INSTRU-1);
	((infos_cred *)instrument->structure)->nom_instru[TAILLE_NOM_INSTRU-1] = NULL;
}

static void cred_configure(PInstrument *cred)
{
	infos_cred *data_cred = cred->structure;
	
	int panel;
	if ((panel = LoadPanel (0, "UIR_cred.uir", Conf_Cred)) < 0)
		return ;
					   
	SetCtrlVal (panel, Conf_Cred_RING_Conf_COM_Cred, data_cred->port_com);
	SetCtrlVal (panel, Conf_Cred_NUM_Conf_Echan_Cred, data_cred->temps_echantillonnage);
	
	SetCtrlAttribute (panel, Conf_Cred_OK_CRED, ATTR_CALLBACK_DATA, data_cred);
	DisplayPanel (panel);
}

static void cred_getconfig(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{
	infos_cred *data = instrument->structure;

	*p_strct = (void*)data;
	*p_member = (void*)&(data->numero_mesure);
	
	*p_bool = 0;
}

static int cred_nb_voie(PInstrument *cred)
{
	infos_cred *data_cred = cred->structure;
	
	return data_cred->nombre_voie;
}

static void cred_get_nom_voie(PInstrument *cred, int voie, char *nom_voie)
{
	sprintf(nom_voie, "Voie %i", voie);
}

static double cred_mesure(PInstrument *cred, int voie)
{
	infos_cred *data_cred = cred->structure;
	
	return data_cred->zone_memoire[voie-1];
}

static unsigned int cred_numero(PInstrument *cred, int voie)
{
	infos_cred *data_cred = cred->structure;
	
	return data_cred->numero_mesure[voie-1];
}

static int cred_digits(PInstrument *cred)
{
	return 9; // ameliorer...
}

int CVICALLBACK CRED_OK (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
int Valeurs_Configuration_Port_COM = 0;
int Valeurs_Configuration_Frequence_Echantillonnage = 0;

infos_cred *structu_inst;

structu_inst = (infos_cred *)callbackData;	

	
	switch (event)
		{
		case EVENT_COMMIT:
		
			GetCtrlVal (panel, Conf_Cred_RING_Conf_COM_Cred, &Valeurs_Configuration_Port_COM);
			GetCtrlVal (panel, Conf_Cred_NUM_Conf_Echan_Cred, &Valeurs_Configuration_Frequence_Echantillonnage);
			
			structu_inst->port_com = Valeurs_Configuration_Port_COM;
			structu_inst->temps_echantillonnage = Valeurs_Configuration_Frequence_Echantillonnage;
			structu_inst->nombre_voie = 4;
			
			DiscardPanel (panel);
			break;
		}
	return 0;
}
