#include "toolbox.h"
#include <rs232.h>
#include <utility.h>
/*
Fichier gnr automatiquement pour l'instrument @
*/

#include <ansi_c.h>
#include <userint.h>
#include <gpib.h>

#include "plugin.h"

#include "UIR_DGM110.h"

#include "DGM110.h"

PInstrument DGM110 =
{
NULL

,NULL
,NULL
,NULL
,NULL
,NULL

// Fonctions obligatoires
,DGM110_nouveau
,DGM110_liberer
,DGM110_configure
,DGM110_get_config
,DGM110_get_nom
,DGM110_set_nom

// Stop, start, init, ect..
,DGM110_stop
,DGM110_start
,DGM110_init
,NULL
,NULL
,NULL

// Mesures 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

// Calculs 1D
,NULL
,NULL
,NULL
,NULL

// Mesures 1D
,DGM110_nb_voies_1D
,DGM110_get_nom_voie_1D
,DGM110_mesure_1D
,DGM110_numero_1D

// Calculs 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

};

//-----------------------------------------------------------------------
// Ajout d'un instrument, remplit la structure PInstrument.
//
// Fonction appelle lors de l'ajout d'un instrument (bouton Ajouter)
// et lors du chargement d'une configuration.
//-----------------------------------------------------------------------
static void DGM110_nouveau( PInstrument** instrument )
{
	
	*instrument = (PInstrument*)malloc(sizeof(PInstrument));
	memcpy(*instrument, &DGM110, sizeof(PInstrument));
	(*instrument)->structure = malloc(sizeof(infos_DGM110));

	sprintf(((infos_DGM110*)(*instrument)->structure)->nom_instrument,"DGM110");

	((infos_DGM110*)(*instrument)->structure)->com_port = 2;
	((infos_DGM110*)(*instrument)->structure)->zone_memoire_mes1D[0]=NotANumber();
	
	return;
}

//-----------------------------------------------------------------------
// Libre la mmoire alloue  l'instrument.
//
// Appelle  la fin du programme.
//
// Jamais de malloc sans son free !
//-----------------------------------------------------------------------
static void DGM110_liberer( PInstrument* instrument )
{
	infos_DGM110* data = instrument->structure;

	free(instrument->structure);
	free(instrument);

	return;
}

//-----------------------------------------------------------------------
// Copie le nom de l'instrument dans char* nom
//-----------------------------------------------------------------------
static void DGM110_get_nom( PInstrument* instrument, char* nom )
{
	infos_DGM110* data = instrument->structure;
	
	memcpy(nom, data->nom_instrument, TAILLE_NOM_INSTRU-1);
	return;
}

//-----------------------------------------------------------------------
// Fixe le nom de l'instrument  char* nom
//-----------------------------------------------------------------------
static void DGM110_set_nom( PInstrument* instrument, char* nom )
{
	infos_DGM110* data = instrument->structure;
	
	memcpy( data->nom_instrument, nom, TAILLE_NOM_INSTRU-1);
	data->nom_instrument[TAILLE_NOM_INSTRU-1] = NULL;
	return;
}

//-----------------------------------------------------------------------
// Appelle la fentre de configuration de l'instrument
//
// On doit fixer les valeurs des controles avant l'apparition de la
// fenetre (cf. commentaires dans le code)
//-----------------------------------------------------------------------
static void DGM110_configure( PInstrument* instrument )
{
	infos_DGM110* data = instrument->structure;

	int panel;
	if ((panel = LoadPanel (0, "UIR_DGM110.uir", P_INSTR)) < 0)
		return ;
	
	// On fixe les valeurs des controles aux valeures courantes.
	
	SetCtrlVal(panel, P_INSTR_RG_COM_ID, data->com_port);

	SetCtrlAttribute (panel, P_INSTR_CB_OK, ATTR_CALLBACK_DATA, data);
	
	DisplayPanel (panel); // Affiche la fentre
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie les paramtres pour sauvegarder la configuration de l'instrument.
//
// !! Ne pas toucher au membre 'dont_remove_me' de la structure      !!
// !! Il sert  diffrencier la partie  sauvegarder de la structure !!
//-----------------------------------------------------------------------
static void DGM110_get_config(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_DGM110* data = instrument->structure;

	*p_strct = (void*)data;
	
	*p_member = (void*)&(data->dont_remove_me);
	
	*p_bool = 0;
	
	return;
}

//-----------------------------------------------------------------------
// Arrte l'acquisition des donnes de l'instrument.
//
// Cette fonction est appelle :
//  -> A la fin d'une acquisition.
//  -> Lors de l'initialisation si un instrument renvoie
//    une erreur, alors tous sont stopps.
//-----------------------------------------------------------------------
static void DGM110_stop( PInstrument* instrument )
{
	infos_DGM110* data = instrument->structure;
	int err;
	
	err = InstallComCallback (data->com_port, 0, 0, 0, DGM_comint, NULL);
}

//-----------------------------------------------------------------------
// Dmarre l'acquisition des donnes de l'instrument
//
// Cette fonction est appelle :
//  -> Au dmarrage d'une acquisition (bouton Acquisition)
//  -> Au redmararge d'une acq. aprs une pause.
//-----------------------------------------------------------------------
static int DGM110_start( PInstrument* instrument )
{
	infos_DGM110* data = instrument->structure;
	int err;

	// On attend le 'E' du message "ETAL: xxxx PPB"
	err = InstallComCallback (data->com_port, 0x0002 ,0,'E', DGM_comint, (void *)data);
	return err;
}

//-----------------------------------------------------------------------
// Initialise l'instrument pour une acquisition
//
// Cette fonction est appelle au dmarrage d'un acquisition (bouton Acquisition)
//-----------------------------------------------------------------------
static int DGM110_init( PInstrument* instrument )
{
	infos_DGM110* data = instrument->structure;

	data->zone_memoire_mes1D[0]=NotANumber();
	data->numero_mes1D[0]=0;
	
	// Ouverture du port srie
	OpenComConfig (data->com_port, NULL, 9600, 0, 8, 1, 512, 512);

	return 0;
}



//-----------------------------------------------------------------------
// Renvoie le nombre de voies de l'instrument.
//-----------------------------------------------------------------------
static int DGM110_nb_voies_1D( PInstrument* instrument )
{
	return NOMBRE_VOIES_MESURE1D;
}

//-----------------------------------------------------------------------
// Renvoie le nom de la voie 'int voie' de l'instrument.
//-----------------------------------------------------------------------
static void DGM110_get_nom_voie_1D( PInstrument* instrument, int voie, char* string )
{

	if (voie == 1)
	{
		sprintf(string,"Concentration");
		return;
	}
	sprintf(string,"Erreur");
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie la mesure courante de la voie 'int voie' de l'instrument.
//-----------------------------------------------------------------------
static double DGM110_mesure_1D( PInstrument* instrument, int voie )
{
	infos_DGM110* data = instrument->structure;

	return data->zone_memoire_mes1D[voie-1];
}

//-----------------------------------------------------------------------
// Renvoie le numro de la mesure courante de la voie 'int voie' de
// l'instrument.
//-----------------------------------------------------------------------
static unsigned int DGM110_numero_1D( PInstrument* instrument, int voie )
{
	infos_DGM110* data = instrument->structure;

	return data->numero_mes1D[voie-1];
}



/*-----------------------------------------------------------------*/
/*              CALLBACKS INTERFACE UTILISATEUR
/*-----------------------------------------------------------------*/

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_DGM110_config_ok(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i_tmp;

	infos_DGM110* data = (infos_DGM110*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:

			// Rcupre le numro du port COM
			GetCtrlVal(panel, P_INSTR_RG_COM_ID, &i_tmp);
			data->com_port = i_tmp;

			DiscardPanel(panel);
			
			break;
		}
	return 0;
}



// Fonction d'interruption sur le port
void CVICALLBACK DGM_comint(int COMPort, int eventMask, void *CallbackData)
{
	infos_DGM110 *data;

	char bidon[2];
	int err,status, concentration,i_tmp;

	data = (infos_DGM110 *) CallbackData; 
	
	if (!(eventMask & 0x0002))
		return;

	if (data==NULL) //Au cas o l'arrt de l'acquisition libre le pointeur et o il y a encore un interruption
		return ;

	// Rcuprer le nombre d'octets dans la file
	i_tmp = GetInQLen (data->com_port);
	if (i_tmp ==0) // l 'interruption qui ne sert  rien.
		Delay(0.1);
	i_tmp = GetInQLen (data->com_port);
	if (i_tmp ==0) // l 'interruption qui ne sert VRAIMENT  rien.
		return;			// on sort
	// arriv ici on a quelque chose dans la file d'attente

	do
	{
		err=ComRdByte (data->com_port);
	} while (err != 'E');
	
	concentration = 0;
	status = 1;
	bidon[1]=0;
	while (status == 1)
	{
		err=ComRdByte (data->com_port);
		bidon[0]=err;

		if (err == 'P')
		{
			status=0;
		}
		else if (isdigit(err) && (status == 1) )
		{
			//i_tmp=atoi(bidon);
			concentration = concentration*10+atoi(bidon);
		}
	}
	FlushInQ(data->com_port);

	// Rcupration des rsultats de mesure
	data->zone_memoire_mes1D[0] = (double) concentration;

	// Incrmentation du nombre de mesures.
	data->numero_mes1D[0]++;   
}
