/*
Fichier gnr automatiquement pour l'instrument Electrovanne@
*/

#include <ansi_c.h>
#include <userint.h>
#include <gpib.h>

#include <utility.h>
#include "plugin.h"

#include "UIR_Electrovanne.h"

#include "Electrovanne.h"

PInstrument Electrovanne =
{
NULL

,NULL
,NULL
,NULL
,NULL
,NULL

// Fonctions obligatoires
,Electrovanne_nouveau
,Electrovanne_liberer
,Electrovanne_configure
,Electrovanne_get_config
,Electrovanne_get_nom
,Electrovanne_set_nom

// Stop, start, init, ect..
,Electrovanne_stop
,Electrovanne_start
,Electrovanne_init
,Electrovanne_status
,Electrovanne_execute
,Electrovanne_piloter

// Mesures 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

// Calculs 1D
,NULL
,NULL
,NULL
,NULL

// Mesures 1D
,NULL
,NULL
,NULL
,NULL

// Calculs 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

};

//-----------------------------------------------------------------------
// Ajout d'un instrument, remplit la structure PInstrument.
//
// Fonction appelle lors de l'ajout d'un instrument (bouton Ajouter)
// et lors du chargement d'une configuration.
//-----------------------------------------------------------------------
static void Electrovanne_nouveau( PInstrument** instrument )
{
	
	*instrument = (PInstrument*)malloc(sizeof(PInstrument));
	memcpy(*instrument, &Electrovanne, sizeof(PInstrument));
	(*instrument)->structure = malloc(sizeof(infos_Electrovanne));

	sprintf(((infos_Electrovanne*)(*instrument)->structure)->nom_instrument,"Electrovanne");

	((infos_Electrovanne*)(*instrument)->structure)->panel_pilotage =0;
	
	// Initialisation de la position de la vanne dans la structure.
	((infos_Electrovanne*)(*instrument)->structure)->etat_vanne =0;

	return;
}

//-----------------------------------------------------------------------
// Libre la mmoire alloue  l'instrument.
//
// Appelle  la fin du programme.
//
// Jamais de malloc sans son free !
//-----------------------------------------------------------------------
static void Electrovanne_liberer( PInstrument* instrument )
{
	infos_Electrovanne* data = instrument->structure;

	DiscardPanel(data->panel_pilotage);

	free(instrument->structure);
	free(instrument);

	return;
}

//-----------------------------------------------------------------------
// Copie le nom de l'instrument dans char* nom
//-----------------------------------------------------------------------
static void Electrovanne_get_nom( PInstrument* instrument, char* nom )
{
	infos_Electrovanne* data = instrument->structure;
	
	memcpy(nom, data->nom_instrument, TAILLE_NOM_INSTRU-1);
	return;
}

//-----------------------------------------------------------------------
// Fixe le nom de l'instrument  char* nom
//-----------------------------------------------------------------------
static void Electrovanne_set_nom( PInstrument* instrument, char* nom )
{
	infos_Electrovanne* data = instrument->structure;
	
	memcpy( data->nom_instrument, nom, TAILLE_NOM_INSTRU-1);
	data->nom_instrument[TAILLE_NOM_INSTRU-1] = NULL;
	return;
}

//-----------------------------------------------------------------------
// Appelle la fentre de configuration de l'instrument
//
// On doit fixer les valeurs des controles avant l'apparition de la
// fenetre (cf. commentaires dans le code)
//-----------------------------------------------------------------------
static void Electrovanne_configure( PInstrument* instrument )
{
	infos_Electrovanne* data = instrument->structure;

	int panel;
	if ((panel = LoadPanel (0, "UIR_Electrovanne.uir", P_INSTR)) < 0)
		return ;
	
	// On fixe les valeurs des controles aux valeures courantes.
	SetCtrlVal(panel, P_INSTR_BS_SIDE, data->etat_vanne);

	SetCtrlAttribute (panel, P_INSTR_CB_OK, ATTR_CALLBACK_DATA, data);
	
	DisplayPanel (panel); // Affiche la fentre
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie les paramtres pour sauvegarder la configuration de l'instrument.
//
// !! Ne pas toucher au membre 'dont_remove_me' de la structure      !!
// !! Il sert  diffrencier la partie  sauvegarder de la structure !!
//-----------------------------------------------------------------------
static void Electrovanne_get_config(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_Electrovanne* data = instrument->structure;

	*p_strct = (void*)data;
	
	*p_member = (void*)&(data->dont_remove_me);
	
	*p_bool = 0;
	
	return;
}

//-----------------------------------------------------------------------
// Arrte l'acquisition des donnes de l'instrument.
//
// Cette fonction est appelle :
//  -> A la fin d'une acquisition.
//  -> Lors de l'initialisation si un instrument renvoie
//    une erreur, alors tous sont stopps.
//-----------------------------------------------------------------------
static void Electrovanne_stop( PInstrument* instrument )
{

	return;
}

//-----------------------------------------------------------------------
// Dmarre l'acquisition des donnes de l'instrument
//
// Cette fonction est appelle :
//  -> Au dmarrage d'une acquisition (bouton Acquisition)
//  -> Au redmararge d'une acq. aprs une pause.
//-----------------------------------------------------------------------
static int Electrovanne_start( PInstrument* instrument )
{

	return 0;
}



//-----------------------------------------------------------------------
// Initialise l'instrument pour une acquisition
//
// Cette fonction est appelle au dmarrage d'un acquisition (bouton Acquisition)
//-----------------------------------------------------------------------
static int Electrovanne_init( PInstrument* instrument )
{
	infos_Electrovanne* data = instrument->structure;


	// Charge le panel de pilotage
	if ( data->panel_pilotage == 0)
		if ( ( data->panel_pilotage = LoadPanel (0, "UIR_Electrovanne.uir", P_PILOT)) < 0)
			return -1;



	return 0;
}

//-----------------------------------------------------------------------
// Renvoie le status de l'instrument.
//
// Les status possibles de l'instrument sont :
//
// STATUS_OQP	0
// STATUS_PRET	1
//-----------------------------------------------------------------------
static int Electrovanne_status( PInstrument* instrument )
{

	return STATUS_PRET;
}

//-----------------------------------------------------------------------
// Execute une commande sur l'instrument.
//
// La syntaxe est la suivante : "sens=x" avec x=0 ou x=1
//
//-----------------------------------------------------------------------
static void Electrovanne_execute( PInstrument* instrument, char* chaine)
{
	infos_Electrovanne *data = instrument->structure;
	char	c_tmp;
	
	if(sscanf(chaine, "sens=%c", &c_tmp) == 1)
	{
		data->etat_vanne=c_tmp;
		// Ecriture directe sur le Port parallle pour commander D0 (broche 2)
		outp(0x378, data->etat_vanne);		
	}
	
	// Mise  jour de l'affichage
	SetCtrlVal(data->panel_pilotage, P_PILOT_BS_SIDE, data->etat_vanne);

	return;
}

//-----------------------------------------------------------------------
// Active le pilotage de l'instrument.
//
// <TODO>
//-----------------------------------------------------------------------
static void Electrovanne_piloter( PInstrument* instrument )
{

	infos_Electrovanne *data = instrument->structure;

	SetCtrlVal(data->panel_pilotage, P_PILOT_BS_SIDE, data->etat_vanne);
	SetCtrlAttribute(data->panel_pilotage, P_PILOT_CB_OK, ATTR_CALLBACK_DATA, data);
	
	DisplayPanel(data->panel_pilotage);

	return;
}



/*-----------------------------------------------------------------*/
/*              CALLBACKS INTERFACE UTILISATEUR
/*-----------------------------------------------------------------*/

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Electrovanne_config_ok(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	//int 	i_tmp;
	char	c_tmp;

	infos_Electrovanne* data = (infos_Electrovanne*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:
			// Inversion du bit de commande de la vanne
			GetCtrlVal(panel, P_INSTR_BS_SIDE, &c_tmp);
			data->etat_vanne=c_tmp;
			
			// Ecriture directe sur le Port parallle pour commander D0 (broche 2)
			outp(0x378, data->etat_vanne);

			DiscardPanel(panel);
			
			break;
		}
	return 0;
}

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'OK' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Electrovanne_pilot_OK (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char	c_tmp;
	
	infos_Electrovanne* data = (infos_Electrovanne*)callbackData;
	
	switch (event)
		{
		case EVENT_COMMIT:
			// Inversion du bit de commande de la vanne
			GetCtrlVal(panel, P_PILOT_BS_SIDE, &c_tmp);
			data->etat_vanne=c_tmp;
			
			// Ecriture directe sur le Port parallle pour commander D0 (broche 2)
			// 0x378 est l'adresse du port, donc ventuellement  modifier sur un PC
			// comportant plusieurs ports parallles...
			outp(0x378, data->etat_vanne);
			break;
		}
	return 0;
}

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Cacher' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Electrovanne_hide(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{

 	switch (event)
	{
		case EVENT_COMMIT:
			HidePanel(panel);
			break;
	}
	
	return 0;
}

