/*
Fichier gnr automatiquement pour l'instrument GBF HP3312A@
*/

#include <ansi_c.h>
#include <userint.h>
#include <gpib.h>

#include "plugin.h"
#include "globales.h"

#include "UIR_GBF_HP3312A.h"
#include "UIR_GBF_HP3312A_pilot.h"

#include "GBF_HP3312A.h"

PInstrument GBF_HP3312A =
{
NULL

,NULL
,NULL
,NULL
,NULL
,NULL

// Fonctions obligatoires
,GBF_HP3312A_nouveau
,GBF_HP3312A_liberer
,GBF_HP3312A_configure
,GBF_HP3312A_get_config
,GBF_HP3312A_get_nom
,GBF_HP3312A_set_nom

// Stop, start, init, ect..
,NULL
,NULL
,GBF_HP3312A_init
,NULL
,GBF_HP3312A_execute
,GBF_HP3312A_piloter

// Mesures 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

// Calculs 1D
,NULL
,NULL
,NULL
,NULL

// Mesures 1D
,NULL
,NULL
,NULL
,NULL

// Calculs 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

};

//-----------------------------------------------------------------------
// Ajout d'un instrument, remplit la structure PInstrument.
//
// Fonction appelle lors de l'ajout d'un instrument (bouton Ajouter)
// et lors du chargement d'une configuration.
//-----------------------------------------------------------------------
static void GBF_HP3312A_nouveau( PInstrument** instrument )
{
	
	*instrument = (PInstrument*)malloc(sizeof(PInstrument));
	memcpy(*instrument, &GBF_HP3312A, sizeof(PInstrument));
	(*instrument)->structure = malloc(sizeof(infos_GBF_HP3312A));

	sprintf(((infos_GBF_HP3312A*)(*instrument)->structure)->nom_instrument,"GBF HP3312A");

	((infos_GBF_HP3312A*)(*instrument)->structure)->panel_pilotage =0;

	
	
	((infos_GBF_HP3312A*)(*instrument)->structure)->gpib_adresse = 1;

	// Imposer les valeurs par dfaut des contrles graphiques.

	return;
}

//-----------------------------------------------------------------------
// Libre la mmoire alloue  l'instrument.
//
// Appelle  la fin du programme.
//
// Jamais de malloc sans son free !
//-----------------------------------------------------------------------
static void GBF_HP3312A_liberer( PInstrument* instrument )
{
	infos_GBF_HP3312A* data = instrument->structure;

	DiscardPanel(data->panel_pilotage);

	free(instrument->structure);
	free(instrument);

	return;
}

//-----------------------------------------------------------------------
// Copie le nom de l'instrument dans char* nom
//-----------------------------------------------------------------------
static void GBF_HP3312A_get_nom( PInstrument* instrument, char* nom )
{
	infos_GBF_HP3312A* data = instrument->structure;
	
	memcpy(nom, data->nom_instrument, TAILLE_NOM_INSTRU-1);
	return;
}

//-----------------------------------------------------------------------
// Fixe le nom de l'instrument  char* nom
//-----------------------------------------------------------------------
static void GBF_HP3312A_set_nom( PInstrument* instrument, char* nom )
{
	infos_GBF_HP3312A* data = instrument->structure;
	
	memcpy( data->nom_instrument, nom, TAILLE_NOM_INSTRU-1);
	data->nom_instrument[TAILLE_NOM_INSTRU-1] = NULL;
	return;
}

//-----------------------------------------------------------------------
// Appelle la fentre de configuration de l'instrument
//
// On doit fixer les valeurs des controles avant l'apparition de la
// fenetre (cf. commentaires dans le code)
//-----------------------------------------------------------------------
static void GBF_HP3312A_configure( PInstrument* instrument )
{
	infos_GBF_HP3312A* data = instrument->structure;

	int panel;
	if ((panel = LoadPanel (0, "UIR_GBF_HP3312A.uir", P_INSTR)) < 0)
		return ;
	
	// On fixe les valeurs des controles aux valeures courantes.
	
	SetCtrlVal(panel, P_INSTR_NM_GPIB_ADRSS, data->gpib_adresse);
	
	SetCtrlAttribute (panel, P_INSTR_CB_OK, ATTR_CALLBACK_DATA, data);
	
	DisplayPanel (panel); // Affiche la fentre
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie les paramtres pour sauvegarder la configuration de l'instrument.
//
// !! Ne pas toucher au membre 'dont_remove_me' de la structure      !!
// !! Il sert  diffrencier la partie  sauvegarder de la structure !!
//-----------------------------------------------------------------------
static void GBF_HP3312A_get_config(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_GBF_HP3312A* data = instrument->structure;

	*p_strct = (void*)data;
	
	*p_member = (void*)&(data->dont_remove_me);
	
	*p_bool = 0;
	
	return;
}





//-----------------------------------------------------------------------
// Initialise l'instrument pour une acquisition
//
// Cette fonction est appelle au dmarrage d'un acquisition (bouton Acquisition)
//-----------------------------------------------------------------------
static int GBF_HP3312A_init( PInstrument* instrument )
{
	infos_GBF_HP3312A* data = instrument->structure;
	int id_cont, err;
	short err_short;
	char chaine[255];


	// Charge le panel de pilotage
	if ( data->panel_pilotage == 0)
		if ( ( data->panel_pilotage = LoadPanel (0, "UIR_GBF_HP3312A_pilot.uir", P_PILOT)) < 0)
			return -1;

	// Positionner les "callbackdata" des contrles du panel pilotage qui ont un callback
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_FORME, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_Burst, ATTR_CALLBACK_DATA, data);
	


	err = ibfind("GPIB0");
	if (err & ERR)
	{
		MessagePopup ("Problme de carte", "Carte GPIB non dtecte");
		return -1;
	}
	
	// Dtection appareil
	id_cont = ibdev (0, data->gpib_adresse, NO_SAD, T10s, 1, 1);
	ibclr (id_cont);		// Erreur CVI si branche mais pas allume !?
	ibln (id_cont, data->gpib_adresse, NO_SAD, &err_short);

	if (err_short == 0)
		{
		sprintf(chaine,"Il faudrait penser  brancher le GBF  l'adresse %d !!!",data->gpib_adresse);
		MessagePopup ("PAS BON", chaine);
		return -1;
		}
	
	ibecr(id_cont, "*IDN?");
	ibrd(id_cont, chaine, 250);
	if (strstr (chaine, "HEWLETT-PACKARD,33220A") == NULL)
	{
		MessagePopup ("Erreur", "Modle non support par ce module !");
		return -1;
	}

	
	// Initialisation de l'instrument
	
	ibecr(id_cont, "APPL:SIN 5 KHZ, 3.0 VPP, 0V");
	
	
	// Mise  jour des donnes de la structure.
	data->identificateur = id_cont;
	
	return 0;
}


//-----------------------------------------------------------------------
// Execute une commande sur l'instrument.
//
// <TODO>
//
//-----------------------------------------------------------------------
static void GBF_HP3312A_execute( PInstrument* instrument, char* string )
{	 double frq, amp, rate;
	 int NbCyc;
	 char Command[100];
	infos_GBF_HP3312A *data = instrument->structure;
	
	//Frequence et amplitude 
	 if (sscanf(string, "frequence=%lf", &frq) == 1)
	{
		
		SetCtrlVal (data->panel_pilotage, P_PILOT_Frequence, frq);
		sprintf(Command,"SOUR:FREQ %lf",frq);
		ibecr(data->identificateur, Command);
	}
	  if (sscanf(string, "amplitude=%lf", &amp) == 1)
	{
		
		SetCtrlVal (data->panel_pilotage, P_PILOT_Amplitude, amp);
		sprintf(Command,"SOUR:VOLT %lf",amp);
		ibecr(data->identificateur, Command);
	}
	
	// Gestion Burst	 
		 if (sscanf(string, "burst rate=%lf", &rate) == 1)
	{
		data->BMRate = rate;
		SetCtrlVal (data->panel_pilotage, P_PILOT_Rate, rate);
		
	}
		 if (sscanf(string, "burst cycle=%d", &NbCyc) == 1)
	{
		data->BMNbCyc = NbCyc;
		SetCtrlVal (data->panel_pilotage, P_PILOT_NbCycle, NbCyc);
	
	}		  
		if (!strcmp(string, "burst ON"))
	{
		sprintf(Command,"BM:NCYC %d",data->BMNbCyc);
		ibecr(data->identificateur, Command);
		sprintf(Command,"BM:INT:RATE %lf",data->BMRate);
		ibecr(data->identificateur, Command);
		ibecr(data->identificateur, "BM:STATE ON");
	}	 
		 if (!strcmp(string, "burst OFF"))
	{
	
		ibecr(data->identificateur, "BM:STATE OFF");
	}	 

	// FORME D ONDE    Interprter "string" et envoyer en consquence  l'instrument.
	if (!strcmp(string, "sinus"))
	{
		SetCtrlVal(data->panel_pilotage, P_PILOT_FORME, 0);
		ibecr(data->identificateur, "SOUR:FUNC:SHAP:SIN");
	}
	else
	if (!strcmp(string, "carre"))
	{
		SetCtrlVal(data->panel_pilotage, P_PILOT_FORME, 1);
		ibecr(data->identificateur, "SOUR:FUNC:SHAP:SQR");
	}
	else
	if (!strcmp(string, "triangle"))
	{
		SetCtrlVal(data->panel_pilotage, P_PILOT_FORME, 2);
		ibecr(data->identificateur, "SOUR:FUNC:SHAP:TRI");
	}
	 else
	if (!strcmp(string, "rampe"))
	{
		SetCtrlVal(data->panel_pilotage, P_PILOT_FORME, 3);
		ibecr(data->identificateur, "SOUR:FUNC:SHAP:RAMP");
	}
	  else
	if (!strcmp(string, "bruit"))
	{
		SetCtrlVal(data->panel_pilotage, P_PILOT_FORME, 4);
		ibecr(data->identificateur, "SOUR:NOIS");
	}
	return;
}

//-----------------------------------------------------------------------
// Active le pilotage de l'instrument.
//
// <TODO>
//-----------------------------------------------------------------------
static void GBF_HP3312A_piloter( PInstrument* instrument )
{

	infos_GBF_HP3312A *data = instrument->structure;
	DisplayPanel(data->panel_pilotage);

	return;
}





/*-----------------------------------------------------------------*/
/*              CALLBACKS INTERFACE UTILISATEUR
/*-----------------------------------------------------------------*/

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_GBF_HP3312A_config_ok(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i_tmp;

	infos_GBF_HP3312A* data = (infos_GBF_HP3312A*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:

			// On rcupre les valeurs des contrles graphiques et on les mmorise dans la structure

			GetCtrlVal(panel, P_INSTR_NM_GPIB_ADRSS, &data->gpib_adresse);
			
			DiscardPanel(panel);
			
			break;
		}
	return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Cacher' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_GBF_HP3312A_hide(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{

 	switch (event)
	{
		case EVENT_COMMIT:
			HidePanel(panel);
			break;
	}
	
	return 0;
}

int CVICALLBACK CB_Forme (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char Command[200];
	double Frq,Amp;
	int i_tmp;
	infos_GBF_HP3312A* data = (infos_GBF_HP3312A*)callbackData;


	switch (event)
		{
		case EVENT_COMMIT:

			GetCtrlVal(panel, control, &i_tmp);
			GetCtrlVal(panel,P_PILOT_Frequence,&Frq);
			GetCtrlVal(panel,P_PILOT_Amplitude,&Amp);	  
			ibecr(data->identificateur,"BM:STAT OFF");
			switch (i_tmp)
			{
			
				
				case 0: // sinus
					sprintf(Command,"APPL:SIN %f ,%f",Frq,Amp);
					ibecr(data->identificateur, Command);
					break;
				case 1: // carr
					sprintf(Command,"APPL:SQU %f ,%f",Frq,Amp);
					ibecr(data->identificateur, Command);
					break;
				case 2: // triangle
					sprintf(Command,"APPL:TRI %f ,%f",Frq,Amp);
					ibecr(data->identificateur, Command);
					break;
				case 3: // rampe
					sprintf(Command,"APPL:RAMP %f ,%f",Frq,Amp);
					ibecr(data->identificateur, Command);
					break;
				case 4: // bruit
					sprintf(Command,"APPL:NOIS %f ,%f",Frq,Amp); 
					ibecr(data->identificateur, Command);
					break;
			};
			
			break;
		}
	return 0;
}





int CVICALLBACK CB_Burst (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	 char CommandNbCyc[200],CommandRate[200];
	 int NbCyc;
	 double Rate;
	infos_GBF_HP3312A* data = (infos_GBF_HP3312A*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:
			ibecr(data->identificateur,"BM:SOUR INT");
			ibecr(data->identificateur,"BM:STAT ON");
			ibecr(data->identificateur, "BM:PHAS 0");  //Phase
			
			GetCtrlVal(panel,P_PILOT_Rate,&Rate);	  // Rate
			sprintf(CommandRate,"BM:INT:RATE %f",Rate); 
			ibecr(data->identificateur, CommandRate);
			
			
			GetCtrlVal(panel,P_PILOT_NbCycle,&NbCyc);	  // Nb Cycle
			sprintf(CommandNbCyc,"BM:NCYC %i",NbCyc); 
			ibecr(data->identificateur, CommandNbCyc);
			 
			break;
		}
	return 0;
}
