#include "toolbox.h"
/*
Fichier gnr automatiquement pour l'instrument HP53131 - phase
*/

#include <ansi_c.h>
#include <userint.h>
#include <gpib.h>

#include "plugin.h"
#include "globales.h"
#include "UIR_HP53131_phase.h"

#include "HP53131_phase.h"

PInstrument HP53131_phase =
{
NULL

,NULL
,NULL
,NULL
,NULL
,NULL

// Fonctions obligatoires
,HP53131_phase_nouveau
,HP53131_phase_liberer
,HP53131_phase_configure
,HP53131_phase_get_config
,HP53131_phase_get_nom
,HP53131_phase_set_nom

// Stop, start, init, ect..
,HP53131_phase_stop
,HP53131_phase_start
,HP53131_phase_init
,NULL
,NULL
,NULL

// Mesures 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

// Calculs 1D
,NULL
,NULL
,NULL
,NULL

// Mesures 1D
,HP53131_phase_nb_voies_1D
,HP53131_phase_get_nom_voie_1D
,HP53131_phase_mesure_1D
,HP53131_phase_numero_1D

// Calculs 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

};

//-----------------------------------------------------------------------
// Ajout d'un instrument, remplit la structure PInstrument.
//
// Fonction appelle lors de l'ajout d'un instrument (bouton Ajouter)
// et lors du chargement d'une configuration.
//-----------------------------------------------------------------------
static void HP53131_phase_nouveau( PInstrument** instrument )
{
	
	*instrument = (PInstrument*)malloc(sizeof(PInstrument));
	memcpy(*instrument, &HP53131_phase, sizeof(PInstrument));
	(*instrument)->structure = malloc(sizeof(infos_HP53131_phase));

	sprintf(((infos_HP53131_phase*)(*instrument)->structure)->nom_instrument,"HP53131 - phase");


	// TODO: initialisation des variables de la structure de l'instrument
	
	((infos_HP53131_phase*)(*instrument)->structure)->gpib_adresse = 1;


	return;
}

//-----------------------------------------------------------------------
// Libre la mmoire alloue  l'instrument.
//
// Appelle  la fin du programme.
//
// Jamais de malloc sans son free !
//-----------------------------------------------------------------------
static void HP53131_phase_liberer( PInstrument* instrument )
{
	infos_HP53131_phase* data = instrument->structure;

	free(instrument->structure);
	free(instrument);

	return;
}

//-----------------------------------------------------------------------
// Copie le nom de l'instrument dans char* nom
//-----------------------------------------------------------------------
static void HP53131_phase_get_nom( PInstrument* instrument, char* nom )
{
	infos_HP53131_phase* data = instrument->structure;
	
	memcpy(nom, data->nom_instrument, TAILLE_NOM_INSTRU-1);
	return;
}

//-----------------------------------------------------------------------
// Fixe le nom de l'instrument  char* nom
//-----------------------------------------------------------------------
static void HP53131_phase_set_nom( PInstrument* instrument, char* nom )
{
	infos_HP53131_phase* data = instrument->structure;
	
	memcpy( data->nom_instrument, nom, TAILLE_NOM_INSTRU-1);
	data->nom_instrument[TAILLE_NOM_INSTRU-1] = NULL;
	return;
}

//-----------------------------------------------------------------------
// Appelle la fentre de configuration de l'instrument
//
// On doit fixer les valeurs des controles avant l'apparition de la
// fenetre (cf. commentaires dans le code)
//-----------------------------------------------------------------------
static void HP53131_phase_configure( PInstrument* instrument )
{
	infos_HP53131_phase* data = instrument->structure;

	int panel;
	if ((panel = LoadPanel (0, "UIR_HP53131_phase.uir", P_INSTR)) < 0)
		return ;
	
	// TODO: fixer les valeurs des controles avec les valeurs courantes.
	
	SetCtrlVal(panel, P_INSTR_NM_GPIB_ADRSS, data->gpib_adresse);

	SetCtrlVal(panel,P_INSTR_RINGSLIDE,data->mode); 
	SetCtrlVal(panel,P_INSTR_AVER_COUN,data->aver_coun);



	SetCtrlAttribute (panel, P_INSTR_CB_OK, ATTR_CALLBACK_DATA, data);
	DisplayPanel (panel); // Affiche la fentre
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie les paramtres pour sauvegarder la configuration de l'instrument.
//
// !! Ne pas toucher au membre 'dont_remove_me' de la structure      !!
// !! Il sert  diffrencier la partie  sauvegarder de la structure !!
//-----------------------------------------------------------------------
static void HP53131_phase_get_config(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_HP53131_phase* data = instrument->structure;

	*p_strct = (void*)data;
	
	*p_member = (void*)&(data->dont_remove_me);
	
	*p_bool = 0;
	
	return;
}

//-----------------------------------------------------------------------
// Arrte l'acquisition des donnes de l'instrument.
//
// Cette fonction est appelle :
//  -> A la fin d'une acquisition.
//  -> Lors de l'initialisation si un instrument renvoie
//    une erreur, alors tous sont stopps.
//-----------------------------------------------------------------------
static void HP53131_phase_stop( PInstrument* instrument )
{
	infos_HP53131_phase *data = instrument->structure;
	
	if (data->stop_interruption == 0) // dj arrt
		return;
		
	//dsactivation de l'interruption
	data->stop_interruption = 0;

	return;
}

//-----------------------------------------------------------------------
// Dmarre l'acquisition des donnes de l'instrument
//
// Cette fonction est appelle :
//  -> Au dmarrage d'une acquisition (bouton Acquisition)
//  -> Au redmararge d'une acq. aprs une pause.
//-----------------------------------------------------------------------
static int HP53131_phase_start( PInstrument* instrument )
{
	infos_HP53131_phase *data_freq = instrument->structure;
	int err=0;
	char chaine[255];
	unsigned short addr_GPIB=0;


	data_freq->numero_mesure=0;


	err=ibwrt(data_freq->identificateur,"*ESE 1",strlen("*ESE 1"));
	// Activation de l'interruption  la fin des mesures sur le frquencemtre
	if ((err = ibwrt (data_freq->identificateur, "*SRE 32",strlen("*SRE 32"))) & ERR) // Activation
		{
		sprintf(chaine,"Impossible d'initialiser l'interruption\nsur le frquencemtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_freq->identificateur,iberr,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		return err;
		}

	// Enlever le bit RQS
	err = ibrsp(data_freq->identificateur,chaine); 
	// Nettoyage du registre d'interruptions
	ibecr(data_freq->identificateur,"*ESR?");
	ibrd(data_freq->identificateur,chaine,10);
	
	// Mise en place de l'interruption
	err = ibnotify (data_freq->identificateur, data_freq->stop_interruption, CB_interruption_phase,data_freq);
	if (err & ERR)
	{
		sprintf(chaine,"Impossible d'initialiser le frquencemtre\nd'adresse %d",addr_GPIB);
		MessagePopup("Attention !!!",chaine);
		return -1;
	}
	// lancement d'une mesure
	if ((err = ibecr(data_freq->identificateur, ":INIT:IMM;*OPC"))& ERR) // dclenchement immdiat
	{
		sprintf(chaine,"Impossible de lancer une mesure\nsur le frquencemtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_freq->identificateur,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		ibnotify (data_freq->identificateur, 0, CB_interruption_phase,data_freq); //Dsactivation de l'interruption
		return err;
	}
	return err;
}


int __stdcall CB_interruption_phase(int boardOrDevice, int localsta, int localerr, long localcntl, void *callbackData)
{
	infos_HP53131_phase* data;
	int err;
	
	char chaine[256];

	switch (localsta)
	{
		case ERR:
			return 0;
		default:
	
			data = (infos_HP53131_phase *)callbackData;

			// Enlever le bit RQS
			err = ibrsp(boardOrDevice,chaine);

			// TODO: Lire la valeur mesuree
	      
			switch(data->mode)
			{
				case 1: ibecr(boardOrDevice, ":CALC3:DATA?");
				break;

				case 0: ibecr(boardOrDevice,":FETC?");
				break;
			}
			ibrd (boardOrDevice, chaine, 255);
			sscanf(chaine,"%lf", &(data->zone_memoire));
			// TODO: La stoquer dans le champ "zone_memoire" de la structure
	
		    // Incrmentation du nombre de mesures.
		    data->numero_mesure++;

			// Nettoyage du registre d'interruptions
			ibecr(boardOrDevice,"*ESR?");
			ibrd(boardOrDevice,chaine,10);

			// TODO: relancer une mesure en lui demandant de signaler la fin de la mesure
			err = ibecr(boardOrDevice,":INIT:IMM;*OPC");
	
			return data->stop_interruption; // Pour ractiver l'interruption
	}
}



//-----------------------------------------------------------------------
// Initialise l'instrument pour une acquisition
//
// Cette fonction est appelle au dmarrage d'un acquisition (bouton Acquisition)
//-----------------------------------------------------------------------
static int HP53131_phase_init( PInstrument* instrument )
{
	infos_HP53131_phase* data = instrument->structure;
	char chaine[255];
	int err=0,id_cont=0;
	short err_short=0;
	char Command[100]; 
	// Initialisation Zone Mmoire.
	data->zone_memoire=NotANumber();

	//Initialisation Numro Mesure.
	data->numero_mesure = 0;

	err = ibfind("GPIB0");
	if (err & ERR)
	{
		MessagePopup ("Problme de carte", "Carte GPIB non dtecte");
		return -1;
	}

	// Dtection multimtre
	id_cont = ibdev (0, data->gpib_adresse, NO_SAD, T10s, 1, 1);
	ibclr (id_cont);		// Erreur CVI si branche mais pas allume !?
	ibln (id_cont, data->gpib_adresse, NO_SAD, &err_short);

	if (err_short == 0)
		{
		sprintf(chaine,"Il faudrait penser  brancher l'appareil  l'adresse %d !!!",data->gpib_adresse);
		MessagePopup ("PAS BON", chaine);
		return -1;
		}
	
	ibecr(id_cont, "*IDN?");
	ibrd(id_cont, chaine, 250);
	if (strstr (chaine, "HEWLETT-PACKARD,5313") == NULL)
	{
		MessagePopup ("Erreur", "Modle non support par ce module !");
		return -1;
	}

	// Mise  jour des donnes de la structure.
	data->identificateur = id_cont;

	
	// TODO: initialiser l'instrument pour l'acquisition souhaite et le mettre en mesure au coup par coup


    //permet d'utiliser le mode de declenchement immediat
    ibecr(data->identificateur,":INIT:CONT OFF");
    
    // Mesure de phase
	ibecr(data->identificateur,":FUNC \"PHASE\""); 
	 
	//Couplage , filtre a 100k, impedance d'entree , attenuation
	ibecr(data->identificateur,":INP1:COUP AC");
	ibecr(data->identificateur,":INP2:COUP AC");
	ibecr(data->identificateur,":INP1:FILT ON");
	ibecr(data->identificateur,":INP2:FILT ON");
	ibecr(data->identificateur,":INP1:IMP 1000000");
	ibecr(data->identificateur,":INP2:IMP 1000000");
	ibecr(data->identificateur,":INP1:ATT 1");
	ibecr(data->identificateur,":INP2:ATT 1");
	//sensitivity , slope, auto triger , level
	ibecr(data->identificateur,":EVENT1:HYST:REL 100");
	ibecr(data->identificateur,":EVENT2:HYST:REL 100");
	ibecr(data->identificateur,":EVENT1:SLOP POS");
	ibecr(data->identificateur,":EVENT2:SLOP POS");
	ibecr(data->identificateur,":EVEN1:LEV:AUTO ON");
	ibecr(data->identificateur,":EVEN2:LEV:AUTO ON"); 
	ibecr(data->identificateur,":EVEN1:LEV:REL 50");
	ibecr(data->identificateur,":EVEN2:LEV:REL 50");
	
	// Mesure d'une valeur ou d'une moyenne
	ibecr(data->identificateur,":TRIG:COUN:AUTO ON");

	//stats commands 
	switch(data->mode)
	{
		case 1 :
			ibecr(data->identificateur,":DISP:TEXT:FEED \"CALC3\"");
			sprintf(Command,":CALC3:AVER:COUN %d",data->aver_coun);
			ibecr(data->identificateur, Command);
			ibecr(data->identificateur,":CALC3:AVER:TYPE MEAN");
			ibecr(data->identificateur,":CALC3:LFIL:STAT OFF");
			ibecr(data->identificateur,":CALC3:AVER:STAT ON");
			break;
		case 0 :
			ibecr(data->identificateur,":CALC3:AVER:STAT OFF");
			break;
	}
	data->stop_interruption = RQS;
	return err;

}


//-----------------------------------------------------------------------
// Renvoie le nombre de voies de l'instrument.
//-----------------------------------------------------------------------
static int HP53131_phase_nb_voies_1D( PInstrument* instrument )
{
	return 1;
}

//-----------------------------------------------------------------------
// Renvoie le nom de la voie 'int voie' de l'instrument.
//-----------------------------------------------------------------------
static void HP53131_phase_get_nom_voie_1D( PInstrument* instrument, int voie, char* string )
{

	switch(voie)
	{
		case 1:
			sprintf(string,"Phase 1 to 2");
			break;
		default:
			sprintf(string,"Erreur");
	}
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie la mesure courante de la voie 'int voie' de l'instrument.
//-----------------------------------------------------------------------
static double HP53131_phase_mesure_1D( PInstrument* instrument, int voie )
{
	infos_HP53131_phase* data = instrument->structure;

	return data->zone_memoire;
}

//-----------------------------------------------------------------------
// Renvoie le numro de la mesure courante de la voie 'int voie' de
// l'instrument.
//-----------------------------------------------------------------------
static unsigned int HP53131_phase_numero_1D( PInstrument* instrument, int voie )
{
	infos_HP53131_phase* data = instrument->structure;

	return data->numero_mesure;
}



/*-----------------------------------------------------------------*/
/*              CALLBACKS INTERFACE UTILISATEUR
/*-----------------------------------------------------------------*/

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_HP53131_phase_config_ok(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i_tmp;

	infos_HP53131_phase* data = (infos_HP53131_phase*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:

			// TODO: recuperer les valeurs du paneau pour les stocker dans la structure
			GetCtrlVal(panel,P_INSTR_AVER_COUN,&(data->aver_coun));
			GetCtrlVal(panel,P_INSTR_RINGSLIDE,&(data->mode)); 
			GetCtrlVal(panel,P_INSTR_NM_GPIB_ADRSS,&(data->gpib_adresse));
			DiscardPanel(panel);
			
			break;
		}
	return 0;
}



