#include <utility.h>
#include <rs232.h>
/*
Fichier gnr automatiquement pour l'instrument Pompe Ismatec@
*/

#include <ansi_c.h>
#include <userint.h>
#include <gpib.h>

#include "plugin.h"

#include "UIR_Pompe_Ismatec.h"

#include "Pompe_Ismatec.h"

PInstrument Pompe_Ismatec =
{
NULL

,NULL
,NULL
,NULL
,NULL
,NULL

// Fonctions obligatoires
,Pompe_Ismatec_nouveau
,Pompe_Ismatec_liberer
,Pompe_Ismatec_configure
,Pompe_Ismatec_get_config
,Pompe_Ismatec_get_nom
,Pompe_Ismatec_set_nom

// Stop, start, init, ect..
,NULL
,NULL
,Pompe_Ismatec_init
,NULL
,Pompe_Ismatec_execute
,Pompe_Ismatec_piloter

// Mesures 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

// Calculs 1D
,NULL
,NULL
,NULL
,NULL

// Mesures 1D
,NULL
,NULL
,NULL
,NULL

// Calculs 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

};

//-----------------------------------------------------------------------
// Ajout d'un instrument, remplit la structure PInstrument.
//
// Fonction appelle lors de l'ajout d'un instrument (bouton Ajouter)
// et lors du chargement d'une configuration.
//-----------------------------------------------------------------------
static void Pompe_Ismatec_nouveau( PInstrument** instrument )
{
	
	*instrument = (PInstrument*)malloc(sizeof(PInstrument));
	memcpy(*instrument, &Pompe_Ismatec, sizeof(PInstrument));
	(*instrument)->structure = malloc(sizeof(infos_Pompe_Ismatec));

	sprintf(((infos_Pompe_Ismatec*)(*instrument)->structure)->nom_instrument,"Pompe Ismatec");
	((infos_Pompe_Ismatec*)(*instrument)->structure)->panel_pilotage =0;
	
	
	((infos_Pompe_Ismatec*)(*instrument)->structure)->diam_tube =152;

	((infos_Pompe_Ismatec*)(*instrument)->structure)->com_port = 1;
	return;
}

//-----------------------------------------------------------------------
// Libre la mmoire alloue  l'instrument.
//
// Appelle  la fin du programme.
//
// Jamais de malloc sans son free !
//-----------------------------------------------------------------------
static void Pompe_Ismatec_liberer( PInstrument* instrument )
{
	infos_Pompe_Ismatec* data = instrument->structure;

	DiscardPanel(data->panel_pilotage);

	free(instrument->structure);
	free(instrument);

	return;
}

//-----------------------------------------------------------------------
// Copie le nom de l'instrument dans char* nom
//-----------------------------------------------------------------------
static void Pompe_Ismatec_get_nom( PInstrument* instrument, char* nom )
{
	infos_Pompe_Ismatec* data = instrument->structure;
	
	memcpy(nom, data->nom_instrument, TAILLE_NOM_INSTRU-1);
	return;
}

//-----------------------------------------------------------------------
// Fixe le nom de l'instrument  char* nom
//-----------------------------------------------------------------------
static void Pompe_Ismatec_set_nom( PInstrument* instrument, char* nom )
{
	infos_Pompe_Ismatec* data = instrument->structure;
	
	memcpy( data->nom_instrument, nom, TAILLE_NOM_INSTRU-1);
	data->nom_instrument[TAILLE_NOM_INSTRU-1] = NULL;
	return;
}

//-----------------------------------------------------------------------
// Appelle la fentre de configuration de l'instrument
//
// On doit fixer les valeurs des controles avant l'apparition de la
// fenetre (cf. commentaires dans le code)
//-----------------------------------------------------------------------
static void Pompe_Ismatec_configure( PInstrument* instrument )
{
	infos_Pompe_Ismatec* data = instrument->structure;

	int panel;
	if ((panel = LoadPanel (0, "UIR_Pompe_Ismatec.uir", P_INSTR)) < 0)
		return ;
	
	// On fixe les valeurs des controles aux valeurs courantes.
	
	SetCtrlVal(panel, P_INSTR_RG_COM_ID, data->com_port);
	SetCtrlVal(panel, P_INSTR_DIAMETRE, data->diam_tube); 

	SetCtrlAttribute (panel, P_INSTR_CB_OK, ATTR_CALLBACK_DATA, data);
	
	DisplayPanel (panel); // Affiche la fentre
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie les paramtres pour sauvegarder la configuration de l'instrument.
//
// !! Ne pas toucher au membre 'dont_remove_me' de la structure      !!
// !! Il sert  diffrencier la partie  sauvegarder de la structure !!
//-----------------------------------------------------------------------
static void Pompe_Ismatec_get_config(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_Pompe_Ismatec* data = instrument->structure;

	*p_strct = (void*)data;
	
	*p_member = (void*)&(data->dont_remove_me);
	
	*p_bool = 0;
	
	return;
}





//-----------------------------------------------------------------------
// Initialise l'instrument pour une acquisition
//
// Cette fonction est appelle au dmarrage d'un acquisition (bouton Acquisition)
//-----------------------------------------------------------------------
static int Pompe_Ismatec_init( PInstrument* instrument )
{
	infos_Pompe_Ismatec* data = instrument->structure;

	char chaine[20];

	// Charge le panel de pilotage
	if ( data->panel_pilotage == 0)
		if ( ( data->panel_pilotage = LoadPanel (0, "UIR_Pompe_Ismatec.uir", P_PILOT)) < 0)
			return -1;

	// Pour que le callback associ au bouton marche
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_OK_DEBIT, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_SENS_ROT, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_ETAT, ATTR_CALLBACK_DATA, data);
	
	// Ouverture du port srie
	OpenComConfig (data->com_port, NULL, 9600, 0, 8, 1, 512, 512);
	// dfinition de l'adresse de la pompe  1
	ComWrt (data->com_port, "@1\r", 3);    
	// Envoi du diamtre intrieur du tube  la pompe
	sprintf (chaine, "1+0%i\r", data->diam_tube);
	ComWrt (data->com_port, chaine, 7);
	// Dfinition du sens de rotation par dfaut
	ComWrt(data->com_port,"1J\r",3);	
	// Dfinition du mode dbit	
	ComWrt(data->com_port,"1M\r",3);	
	
	return 0;
}


//-----------------------------------------------------------------------
// Execute une commande sur l'instrument.
//
// <TODO>
//
//-----------------------------------------------------------------------
static void Pompe_Ismatec_execute( PInstrument* instrument, char* string )
{
	// Interprtation de 'chaine'
	// Envoi vers l'appareil
	// Mise  jour du panel pour le pilotage manuel
	double valeur;
	float debit_max,tourmin;
	char chaine[16];
	infos_Pompe_Ismatec* data = instrument->structure;
	
	FlushInQ(data->com_port);

	if (!strcmp(string, "marche"))
	{
		SetCtrlVal (data->panel_pilotage, P_PILOT_ETAT, 1);
		sprintf(chaine, "1H\r");
		ComWrt (data->com_port, chaine, strlen(chaine));
		SetCtrlVal (data->panel_pilotage, P_PILOT_LED, 1);
		ComRdByte (data->com_port);
	}
	else
	if  (!strcmp(string, "arret"))
	{
		SetCtrlVal (data->panel_pilotage, P_PILOT_ETAT, 0);
		sprintf(chaine, "1I\r");
		ComWrt (data->com_port, chaine, strlen(chaine));
		SetCtrlVal (data->panel_pilotage, P_PILOT_LED, 0);
		ComRdByte (data->com_port);
	}
	else
	if (!strcmp(string, "sens horaire"))
	{
		SetCtrlVal (data->panel_pilotage, P_PILOT_SENS_ROT, 1);
		sprintf(chaine, "1J\r");
		ComWrt (data->com_port, chaine, strlen(chaine));
		ComRdByte (data->com_port);
	}
	else
	if (!strcmp(string, "sens anti horaire"))
	{
		SetCtrlVal (data->panel_pilotage, P_PILOT_SENS_ROT, 0);
		sprintf(chaine, "1K\r");
		ComWrt (data->com_port, chaine, strlen(chaine));
		ComRdByte (data->com_port);
	}
	if (sscanf(string, "debit=%lf", &valeur) == 1)
	{
		
		SetCtrlVal(data->panel_pilotage, P_PILOT_DEBIT, valeur);
		data->debit = valeur;

		// demande  la pompe le dbit calibr  vitesse max (100tr/min)
		ComWrt (data->com_port, "1!\r", 3);
		// lit la rponse de la pompe
		ComRd (data->com_port, chaine, 15);
		sscanf(chaine,"%f ml/min", &debit_max);
		//calcul du nombre de tours/min  fonction du dbit
		tourmin=(data->debit*100)/debit_max;
		//envoi tours/min  la pompe ( multiplier par 10 d'aprs la doc)
		sprintf(chaine,"1S%05d\r",(int)(10*tourmin));
		ComWrt (data->com_port, chaine, strlen(chaine));
		ComRdByte (data->com_port);
	}
	
	return;
}

//-----------------------------------------------------------------------
// Active le pilotage de l'instrument.
//
// <TODO>
//-----------------------------------------------------------------------
static void Pompe_Ismatec_piloter( PInstrument* instrument )
{

	infos_Pompe_Ismatec *data = instrument->structure;
	DisplayPanel(data->panel_pilotage);

	return;
}





/*-----------------------------------------------------------------*/
/*              CALLBACKS INTERFACE UTILISATEUR
/*-----------------------------------------------------------------*/

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Pompe_Ismatec_config_ok(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i_tmp;
	int diam_tmp;

	infos_Pompe_Ismatec* data = (infos_Pompe_Ismatec*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:

			// Rcupre le numro du port COM
			GetCtrlVal(panel, P_INSTR_RG_COM_ID, &i_tmp);
			data->com_port = i_tmp;
			// Rcupre le diamtre du tube
		    GetCtrlVal(panel, P_INSTR_DIAMETRE, &diam_tmp);
		    data->diam_tube = diam_tmp;
			DiscardPanel(panel);
			
			break;
		}
	return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Cacher' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Pompe_Ismatec_hide(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{

 	switch (event)
	{
		case EVENT_COMMIT:
			HidePanel(panel);
			break;
	}
	
	return 0;
}

// Bouton du dbit dans le panneau de pilotage
int CVICALLBACK CB_debit_changed(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{
	// Ne pas oublier dans l'init de changer l'attribut ATTR_CALLBACK_DATA
	infos_Pompe_Ismatec* data = (infos_Pompe_Ismatec*)callbackData;
	
	double d_tmp, tourmin;
	float debit_max;
	char chaine[50];
	
 	switch (event)
	{
		case EVENT_COMMIT:
			GetCtrlVal(panel, P_PILOT_DEBIT, &d_tmp);
			data->debit = d_tmp;
			
			FlushInQ(data->com_port);
			
			// demande  la pompe le dbit calibr  vitesse max (100tr/min)
			ComWrt (data->com_port, "1!\r", 3);
			// lit la rponse de la pompe
			ComRd (data->com_port, chaine, 15);
			sscanf(chaine,"%f ml/min", &debit_max);
			//calcul du nombre de tours/min  fonction du dbit
			tourmin=(data->debit*100)/debit_max;
			//envoi tours/min  la pompe ( multiplier par 10 d'aprs la doc)
			sprintf(chaine,"1S%05d\r",(int)(10*tourmin));
			ComWrt (data->com_port, chaine, strlen(chaine));
			
			
			break;
	}
	
	return 0;
}

int CVICALLBACK sensos_rot (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int sens;
	char chaine[5];
	
	infos_Pompe_Ismatec* data = (infos_Pompe_Ismatec*)callbackData;
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panel, P_PILOT_SENS_ROT, &sens);
			
			if (sens==1)
				sprintf(chaine, "1J\r");
			else
				sprintf(chaine, "1K\r");
			
			ComWrt (data->com_port, chaine, strlen(chaine));
			break;
		}
	return 0;
}



int CVICALLBACK ETAT_POMPE (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int etat;
	char chaine[5];
	
	infos_Pompe_Ismatec* data = (infos_Pompe_Ismatec*)callbackData;
	
	switch (event)
		{
		case EVENT_COMMIT:
		GetCtrlVal (panel, P_PILOT_ETAT, &etat);
			
			if (etat==1)
				sprintf(chaine, "1H\r");
			else
				sprintf(chaine, "1I\r");
			
			ComWrt (data->com_port, chaine, strlen(chaine));
			SetCtrlVal (panel, P_PILOT_LED, etat);
			break;
		}
	return 0;
}
