/*
Fichier gnr automatiquement pour l'instrument Pousse Seringue@
*/

#include <ansi_c.h>
#include <userint.h>
#include <rs232.h>  
#include "plugin.h"
#include "UIR_Pousse_Seringue.h"
#include "Pousse_Seringue.h"

//#define DEBUG_RS232

PInstrument Pousse_Seringue =
{
NULL

,NULL
,NULL
,NULL
,NULL
,NULL

// Fonctions obligatoires
,Pousse_Seringue_nouveau
,Pousse_Seringue_liberer
,Pousse_Seringue_configure
,Pousse_Seringue_get_config
,Pousse_Seringue_get_nom
,Pousse_Seringue_set_nom

// Stop, start, init, ect..
,Pousse_Seringue_stop
,Pousse_Seringue_start
,Pousse_Seringue_init
,Pousse_Seringue_status
,Pousse_Seringue_execute
,Pousse_Seringue_piloter

// Mesures 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

// Calculs 1D
,NULL
,NULL
,NULL
,NULL

// Mesures 1D
,NULL
,NULL
,NULL
,NULL

// Calculs 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

};

//-----------------------------------------------------------------------
// Ajout d'un instrument, remplit la structure PInstrument.
//
// Fonction appelle lors de l'ajout d'un instrument (bouton Ajouter)
// et lors du chargement d'une configuration.
//-----------------------------------------------------------------------
static void Pousse_Seringue_nouveau( PInstrument** instrument )
{
	int	i=0;
	char 	direction[32]= "DIR INF";
	int		instr_height[4] = {170,300,430,560};
	int		pilot_height[4] = {130,240,350,460};
	*instrument = (PInstrument*)malloc(sizeof(PInstrument));
	memcpy(*instrument, &Pousse_Seringue, sizeof(PInstrument));
	(*instrument)->structure = malloc(sizeof(infos_Pousse_Seringue));

	sprintf(((infos_Pousse_Seringue*)(*instrument)->structure)->nom_instrument,"Pousse Seringue");
	
	((infos_Pousse_Seringue*)(*instrument)->structure)->panel_pilotage=0;
	((infos_Pousse_Seringue*)(*instrument)->structure)->com_port = 2;
	((infos_Pousse_Seringue*)(*instrument)->structure)->nb_PS = 1;


	for(i=0;i<4;i++)
	{
		sprintf(((infos_Pousse_Seringue*)(*instrument)->structure)->label_PS[i],"Pousse Seringue %2d",i+1); 
		((infos_Pousse_Seringue*)(*instrument)->structure)->direction[i] = 0;
		((infos_Pousse_Seringue*)(*instrument)->structure)->diametre[i] = 12.06;
		((infos_Pousse_Seringue*)(*instrument)->structure)->rate[i] = 1000;
		((infos_Pousse_Seringue*)(*instrument)->structure)->adresse[i] = i;
		((infos_Pousse_Seringue*)(*instrument)->structure)->instr_panel_height[i] = instr_height[i];
		((infos_Pousse_Seringue*)(*instrument)->structure)->pilot_panel_height[i] = pilot_height[i];
	}

	return;
}

//-----------------------------------------------------------------------
// Libre la mmoire alloue  l'instrument.
//
// Appelle  la fin du programme.
//
// Jamais de malloc sans son free !
//-----------------------------------------------------------------------
static void Pousse_Seringue_liberer( PInstrument* instrument )
{
	infos_Pousse_Seringue* data = instrument->structure;

	// Fermeture du port srie
	CloseCom(data->com_port);
	
	DiscardPanel(data->panel_pilotage);

	free(instrument->structure);
	free(instrument);

	return;
}


//-----------------------------------------------------------------------
// Copie le nom de l'instrument dans char* nom
//-----------------------------------------------------------------------
static void Pousse_Seringue_get_nom( PInstrument* instrument, char* nom )
{
	infos_Pousse_Seringue* data = instrument->structure;
	
	memcpy(nom, data->nom_instrument, TAILLE_NOM_INSTRU-1);
	return;
}


//-----------------------------------------------------------------------
// Fixe le nom de l'instrument  char* nom
//-----------------------------------------------------------------------
static void Pousse_Seringue_set_nom( PInstrument* instrument, char* nom )
{
	infos_Pousse_Seringue* data = instrument->structure;
	
	memcpy( data->nom_instrument, nom, TAILLE_NOM_INSTRU-1);
	data->nom_instrument[TAILLE_NOM_INSTRU-1] = NULL;
	return;
}


//-----------------------------------------------------------------------
// Appelle la fentre de configuration de l'instrument
//
// On doit fixer les valeurs des controles avant l'apparition de la
// fenetre (cf. commentaires dans le code)
//-----------------------------------------------------------------------
static void Pousse_Seringue_configure( PInstrument* instrument )
{
	infos_Pousse_Seringue* data = instrument->structure;

	int panel, i;
	if ((panel = LoadPanel (0, "UIR_Pousse_Seringue.uir", P_INSTR)) < 0)
		return ;
	
	// On fixe les valeurs des controles aux valeurs courantes.
	SetCtrlVal(panel, P_INSTR_RG_COM_ID, data->com_port);
	SetCtrlVal(panel, P_INSTR_NUM_PS_ID, data->nb_PS);
	// Dimensionnement du panel pour un pousse seringue
	SetPanelSize(panel, data->instr_panel_height[data->nb_PS-1], 280);	
	
	SetCtrlVal(panel, P_INSTR_STR_PS_ID_1, data->label_PS[0]);
	SetCtrlVal(panel, P_INSTR_RG_DIR_ID_1, data->direction[0]);		
	SetCtrlVal(panel, P_INSTR_NUM_DIA_ID_1, data->diametre[0]);	
	SetCtrlVal(panel, P_INSTR_NUM_RAT_ID_1, data->rate[0]);
	SetCtrlVal(panel, P_INSTR_NUM_PS_ID_1, data->adresse[0]);

	SetCtrlVal(panel, P_INSTR_STR_PS_ID_2, data->label_PS[1]);
	SetCtrlVal(panel, P_INSTR_RG_DIR_ID_2, data->direction[1]);		
	SetCtrlVal(panel, P_INSTR_NUM_DIA_ID_2, data->diametre[1]);	
	SetCtrlVal(panel, P_INSTR_NUM_RAT_ID_2, data->rate[1]);
	SetCtrlVal(panel, P_INSTR_NUM_PS_ID_2, data->adresse[1]);

	SetCtrlVal(panel, P_INSTR_STR_PS_ID_3, data->label_PS[2]);
	SetCtrlVal(panel, P_INSTR_RG_DIR_ID_3, data->direction[2]);		
	SetCtrlVal(panel, P_INSTR_NUM_DIA_ID_3, data->diametre[2]);	
	SetCtrlVal(panel, P_INSTR_NUM_RAT_ID_3, data->rate[2]);
	SetCtrlVal(panel, P_INSTR_NUM_PS_ID_3, data->adresse[2]);	

	SetCtrlVal(panel, P_INSTR_STR_PS_ID_4, data->label_PS[3]);
	SetCtrlVal(panel, P_INSTR_RG_DIR_ID_4, data->direction[3]);		
	SetCtrlVal(panel, P_INSTR_NUM_DIA_ID_4, data->diametre[3]);	
	SetCtrlVal(panel, P_INSTR_NUM_RAT_ID_4, data->rate[3]);
	SetCtrlVal(panel, P_INSTR_NUM_PS_ID_4, data->adresse[3]);	
	
	SetCtrlAttribute (panel, P_INSTR_CB_OK, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute(panel, P_INSTR_NUM_PS_ID, ATTR_CALLBACK_DATA, data);
	DisplayPanel (panel); // Affiche la fentre
	
	return;
}


//-----------------------------------------------------------------------
// Renvoie les paramtres pour sauvegarder la configuration de l'instrument.
//
// !! Ne pas toucher au membre 'dont_remove_me' de la structure      !!
// !! Il sert  diffrencier la partie  sauvegarder de la structure !!
//-----------------------------------------------------------------------
static void Pousse_Seringue_get_config(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_Pousse_Seringue* data = instrument->structure;

	*p_strct = (void*)data;
	
	*p_member = (void*)&(data->dont_remove_me);
	
	*p_bool = 0;
	
	return;
}


//-----------------------------------------------------------------------
// Arrte l'acquisition des donnes de l'instrument.
//
// Cette fonction est appelle :
//  -> A la fin d'une acquisition.
//  -> Lors de l'initialisation si un instrument renvoie
//    une erreur, alors tous sont stopps.
//-----------------------------------------------------------------------
static void Pousse_Seringue_stop( PInstrument* instrument )
{

	return;
}


//-----------------------------------------------------------------------
// Dmarre l'acquisition des donnes de l'instrument
//
// Cette fonction est appelle :
//  -> Au dmarrage d'une acquisition (bouton Acquisition)
//  -> Au redmararge d'une acq. aprs une pause.
//-----------------------------------------------------------------------
static int Pousse_Seringue_start( PInstrument* instrument )
{
	infos_Pousse_Seringue* data = instrument->structure;
	
	return 0;
}



//-----------------------------------------------------------------------
// Initialise l'instrument pour une acquisition
//
// Cette fonction est appelle au dmarrage d'une acquisition (bouton Acquisition)
//-----------------------------------------------------------------------
static int Pousse_Seringue_init( PInstrument* instrument )
{
	int		err,i=0;
	char	c_tmp;
	char cmdToSend[32];
	infos_Pousse_Seringue* data = instrument->structure;

	// Charge le panel de pilotage
	if ( data->panel_pilotage == 0)
		if ( ( data->panel_pilotage = LoadPanel (0, "UIR_Pousse_Seringue.uir", P_PILOT)) < 0)
			return -1;
	
	// Dimensionnement du panneau de pilotage en fonction du nombre de pousses seringues
	SetPanelSize(data->panel_pilotage, data->pilot_panel_height[data->nb_PS-1], 280);			

	// Ouverture du port COM RS232
	err=OpenComConfig (data->com_port, NULL, 9600, 0, 8, 1, 512, 512);
	if(err < 0)
	{
		MessagePopup ("Attention", "Impossible d'ouvrir le port demand");
		return err;
	}
	SetXMode(data->com_port, 0);	// Dsactivation du controle de flux logiciel
	SetCTSMode(data->com_port, 0);	// Dsactivation du controle de flux matriel
	SetComTime(data->com_port, 5);	// Activation du timeout en lecture/criture

	// Peut tre utile pour dbloquer lors d'une mauvaise interprtation de la commande
	// Permet galement de tester que le pousse seringue est aliment et connect au PC
	write_read_monitor_rs232(data, "00DIA");

	// Envoi des paramtres d'initialisation des pousses seringues
	// Sens 
	for(i=0;i<data->nb_PS;i++)
	{
		if(data->direction[i] == 0) 
		{
			sprintf(cmdToSend, "%02dDIR INF",data->adresse[i]);
		}
		else sprintf(cmdToSend, "%02dDIR WDR",data->adresse[i]);
	
		write_read_monitor_rs232(data, cmdToSend);
	
		// Diamtre des seringues en mm
		sprintf(cmdToSend, "%02dDIA %.2f",data->adresse[i],data->diametre[i]);
		write_read_monitor_rs232(data, cmdToSend);
	
		// Dbit en L/hr
		sprintf(cmdToSend, "%02dRAT %dUH",data->adresse[i],data->rate[i]);
		write_read_monitor_rs232(data, cmdToSend);
	}

	return 0;
}

//-----------------------------------------------------------------------
// Renvoie le status de l'instrument.
//
// Les status possibles de l'instrument sont :
//
// STATUS_OQP	0
// STATUS_PRET	1
//-----------------------------------------------------------------------
static int Pousse_Seringue_status( PInstrument* instrument )
{

	return STATUS_PRET;
}

//-----------------------------------------------------------------------
// Execute une commande sur l'instrument.
//
// Liste des commandes associes au pousse seringue :
// 		. XXinfuse@XXXXUH, ou les 2 premiers XX reprsentent le numro 
// du pousse seringue (et non son adresse), et les XXXX suivants le dbit 
// exprim en L/hr.
//		. XXstop, ou les 2 premiers XX reprsentent le numro du pousse 
// seringue.
//
// REMARQUES : La numrotation des pousse seringues commence  0. L'affichage
// du dbit n'est pas mis  jour.
//
//-----------------------------------------------------------------------
static void Pousse_Seringue_execute( PInstrument* instrument, char* chaine )
{
	int		i_index, i_rate;
	char	cmdToSend[32];
	
	infos_Pousse_Seringue *data = instrument->structure;
	

	if(sscanf(chaine,"%02dinfuse=%dUH",&i_index, &i_rate)==2)
	{
		// Changement du dbit
		sprintf(cmdToSend, "%02dRAT %dUH",data->adresse[i_index],i_rate);
		write_read_monitor_rs232(data, cmdToSend);						
					

		// Mise en marche
		sprintf(cmdToSend, "%02dRUN",data->adresse[i_index]);
		write_read_monitor_rs232(data, cmdToSend);
		
		// Raffraichissement de l'affichage et du contenu de la structure			
		SetCtrlVal(data->panel_pilotage, index_to_controlID(i_index), 1);
		data->rate[i_index]=i_rate;
		
	}
	else 
	if(sscanf(chaine,"%02dstop",&i_index)==1)
	{
		// Arret du pousse seringue
		sprintf(cmdToSend, "%02dSTP", data->adresse[i_index]);
		write_read_monitor_rs232(data, cmdToSend);
	
		// Raffraichissement de l'affichage et du contenu de la structure			
		SetCtrlVal(data->panel_pilotage, index_to_controlID(i_index), 0);			
	}

	return;
}

//-----------------------------------------------------------------------
// Active le pilotage de l'instrument.
//
// <TODO>
//-----------------------------------------------------------------------
static void Pousse_Seringue_piloter( PInstrument* instrument )
{
	infos_Pousse_Seringue *data = instrument->structure;

	SetCtrlVal(data->panel_pilotage, P_PILOT_LB_ID_1, data->label_PS[0]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_NUM_RAT_ID_1, data->rate[0]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_RG_DIR_ID_1, data->direction[0]);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_RUN_ID_1, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_STP_ID_1, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_RG_DIR_ID_1, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_NUM_RAT_ID_1, ATTR_CALLBACK_DATA, data);	

	SetCtrlVal(data->panel_pilotage, P_PILOT_LB_ID_2, data->label_PS[1]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_NUM_RAT_ID_2, data->rate[1]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_RG_DIR_ID_2, data->direction[1]);	
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_RUN_ID_2, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_STP_ID_2, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_RG_DIR_ID_2, ATTR_CALLBACK_DATA, data);	
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_NUM_RAT_ID_2, ATTR_CALLBACK_DATA, data);	
	
	SetCtrlVal(data->panel_pilotage, P_PILOT_LB_ID_3, data->label_PS[2]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_NUM_RAT_ID_3, data->rate[2]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_RG_DIR_ID_3, data->direction[2]);	
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_RUN_ID_3, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_STP_ID_3, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_RG_DIR_ID_3, ATTR_CALLBACK_DATA, data);	
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_NUM_RAT_ID_3, ATTR_CALLBACK_DATA, data);	

	SetCtrlVal(data->panel_pilotage, P_PILOT_LB_ID_4, data->label_PS[3]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_NUM_RAT_ID_4, data->rate[3]);
	SetCtrlVal(data->panel_pilotage, P_PILOT_RG_DIR_ID_4, data->direction[3]);	
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_RUN_ID_4, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_STP_ID_4, ATTR_CALLBACK_DATA, data);
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_RG_DIR_ID_4, ATTR_CALLBACK_DATA, data);	
	SetCtrlAttribute (data->panel_pilotage, P_PILOT_NUM_RAT_ID_4, ATTR_CALLBACK_DATA, data);	

	SetCtrlAttribute (data->panel_pilotage, P_PILOT_CB_HIDE, ATTR_CALLBACK_DATA, data);	

	DisplayPanel(data->panel_pilotage);

	return;
}


/*-----------------------------------------------------------------*/
/*              CALLBACKS INTERFACE UTILISATEUR
/*-----------------------------------------------------------------*/


//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Pousse_Seringue_nb_PS(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	char	c_tmp;
	
	infos_Pousse_Seringue* data = (infos_Pousse_Seringue*)callbackData;
	
	switch(event)
	{
		case EVENT_COMMIT:
			GetCtrlVal(panel, P_INSTR_NUM_PS_ID, &c_tmp);
			data->nb_PS=c_tmp;
			SetPanelSize(panel, data->instr_panel_height[data->nb_PS-1], 280);
			break;
	}
	
	return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Pousse_Seringue_config_ok(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int 	i_tmp;
	char	c_tmp;
	char	s_tmp[64];
	float   f_tmp;

	infos_Pousse_Seringue* data = (infos_Pousse_Seringue*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:
			// Rcupre le numro du port COM
			GetCtrlVal(panel, P_INSTR_RG_COM_ID, &i_tmp);
			data->com_port = i_tmp;

			// Rcupre les adresses
			GetCtrlVal(panel, P_INSTR_NUM_PS_ID_1, &c_tmp);
			data->adresse[0] = c_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_PS_ID_2, &c_tmp);
			data->adresse[1] = c_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_PS_ID_3, &c_tmp);
			data->adresse[2] = c_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_PS_ID_4, &c_tmp);
			data->adresse[3] = c_tmp;
			
			// Rcupre les labels
			GetCtrlVal(panel, P_INSTR_STR_PS_ID_1, s_tmp);
			sprintf(data->label_PS[0],s_tmp);
			GetCtrlVal(panel, P_INSTR_STR_PS_ID_2, s_tmp);
			sprintf(data->label_PS[1],s_tmp);
			GetCtrlVal(panel, P_INSTR_STR_PS_ID_3, s_tmp);
			sprintf(data->label_PS[2],s_tmp);
			GetCtrlVal(panel, P_INSTR_STR_PS_ID_4, s_tmp);
			sprintf(data->label_PS[3],s_tmp);
			
			// Rcupre le sens
			GetCtrlVal(panel, P_INSTR_RG_DIR_ID_1, &c_tmp);
			data->direction[0] = c_tmp;
			GetCtrlVal(panel, P_INSTR_RG_DIR_ID_2, &c_tmp);
			data->direction[1] = c_tmp;
			GetCtrlVal(panel, P_INSTR_RG_DIR_ID_3, &c_tmp);
			data->direction[2] = c_tmp;
			GetCtrlVal(panel, P_INSTR_RG_DIR_ID_4, &c_tmp);
			data->direction[3] = c_tmp;

	
			// Rcupre le diamtre en mm
			GetCtrlVal(panel, P_INSTR_NUM_DIA_ID_1, &f_tmp);
			data->diametre[0] = f_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_DIA_ID_2, &f_tmp);
			data->diametre[1] = f_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_DIA_ID_3, &f_tmp);
			data->diametre[2] = f_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_DIA_ID_4, &f_tmp);
			data->diametre[3] = f_tmp;
			
	
			// Rcupre le dbit en L/hr
			GetCtrlVal(panel, P_INSTR_NUM_RAT_ID_1, &i_tmp);
			data->rate[0] = i_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_RAT_ID_2, &i_tmp);
			data->rate[1] = i_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_RAT_ID_3, &i_tmp);
			data->rate[2] = i_tmp;
			GetCtrlVal(panel, P_INSTR_NUM_RAT_ID_4, &i_tmp);
			data->rate[3] = i_tmp;

			DiscardPanel(panel);
			
			break;
		}
	return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Cacher' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Pousse_Seringue_pilot_hide(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			HidePanel(panel);
			break;
		}
	return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour l'ascenseur 'DIRECTION' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK RG_Pousse_Seringue_pilot_DIR(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{
	int		index;
	char	c_tmp;
	char	cmdToSend[32];
	infos_Pousse_Seringue* data = (infos_Pousse_Seringue*)callbackData;
	
	switch(event)
   	{
   		case EVENT_COMMIT:
			// Rcupre le sens
			GetCtrlVal(panel, control, &c_tmp);
			index=controlID_to_index(control);
			data->direction[index] = c_tmp;
			if(data->direction[index] == 0) 
			{
				sprintf(cmdToSend, "%02dDIR INF",data->adresse[index]);
			}
			else sprintf(cmdToSend, "%02dDIR WDR",data->adresse[index]);
	
			write_read_monitor_rs232(data, cmdToSend);
			break;
	}
   
   return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour les controles numriques 'RATE' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK NUM_Pousse_Seringue_pilot_RAT(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{
	int		i_tmp, index;
	infos_Pousse_Seringue* data = (infos_Pousse_Seringue*)callbackData;
	
	switch(event)
   	{
   		case EVENT_COMMIT:
			// Rcupre le dbit en L/hr
			GetCtrlVal(panel, control, &i_tmp);
			index=controlID_to_index(control);
			data->rate[index] = i_tmp;
			break;
	}
   
   return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'PUMP' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Pousse_Seringue_pilot_RUN(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{
	int		i_tmp,index;
	char 	cmdToSend[32];
	infos_Pousse_Seringue* data = (infos_Pousse_Seringue*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:
			// Changement du dbit
			index=controlID_to_index(control);
			sprintf(cmdToSend, "%02dRAT %dUH",data->adresse[index],data->rate[index]);
			write_read_monitor_rs232(data, cmdToSend);						
						
			// Mise en marche
			sprintf(cmdToSend, "%02dRUN",data->adresse[index]);
			write_read_monitor_rs232(data, cmdToSend);
			
			// Allumage de la LED associe au pousse seringue			
			SetCtrlVal(panel, index_to_controlID(index), 1);			
			break;
		}
	return 0;
}



//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'STOP' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Pousse_Seringue_pilot_STP(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{
	int	index;
	char cmdToSend[32];
	infos_Pousse_Seringue* data = (infos_Pousse_Seringue*)callbackData;

	switch(event)
	{
		case EVENT_COMMIT:
			// Arret
			index=controlID_to_index(control);
			sprintf(cmdToSend, "%02dSTP",data->adresse[index]);
			write_read_monitor_rs232(data, cmdToSend);

			// Extinction de la LED associe au pousse seringue			
			SetCtrlVal(panel, index_to_controlID(index), 0);			
			break;
	}
	
	return 0;
}


//-----------------------------------------------------------------------
// Fonction utilise pour dterminer l'adresse du pousse seringue en fonction
// de l'ID du control associ
//-----------------------------------------------------------------------
int controlID_to_index(int control_ID)
{
	switch(control_ID)
	{
		case P_PILOT_RG_DIR_ID_1: return 0;
		case P_PILOT_RG_DIR_ID_2: return 1;
		case P_PILOT_RG_DIR_ID_3: return 2;
		case P_PILOT_RG_DIR_ID_4: return 3;
		
		case P_PILOT_NUM_RAT_ID_1: return 0;
		case P_PILOT_NUM_RAT_ID_2: return 1;
		case P_PILOT_NUM_RAT_ID_3: return 2;
		case P_PILOT_NUM_RAT_ID_4: return 3;

		case P_PILOT_CB_RUN_ID_1: return 0;
		case P_PILOT_CB_RUN_ID_2: return 1;
		case P_PILOT_CB_RUN_ID_3: return 2;
		case P_PILOT_CB_RUN_ID_4: return 3;
		
		case P_PILOT_CB_STP_ID_1: return 0;
		case P_PILOT_CB_STP_ID_2: return 1;
		case P_PILOT_CB_STP_ID_3: return 2;
		case P_PILOT_CB_STP_ID_4: return 3;

		default: return -1;
	}
}


//-----------------------------------------------------------------------
// Fonction utilise pour obtenir l'ID de la LED associe aux boutons PUMP
// & STOP
//-----------------------------------------------------------------------
int index_to_controlID(int index)
{
	switch(index)
	{
		case 0: return P_PILOT_LED_RUN_ID_1;
		case 1: return P_PILOT_LED_RUN_ID_2;
		case 2: return P_PILOT_LED_RUN_ID_3;
		case 3: return P_PILOT_LED_RUN_ID_4;
		
		default: return -1;
	}
}


//-----------------------------------------------------------------------
// Fonction du pousse-seringue : envoi de commandes sur la liaison RS232
// Possibilit de scruter les paquets entrants et sortants pour le dbogage.
// Dans l'tat actuel il n'y a pas de controle des paquets de retour.
//
//-----------------------------------------------------------------------
void write_read_monitor_rs232(infos_Pousse_Seringue * data, char * cmdToSend_param)
{
	int	adr_valid=0;
	int i=0, err, nb, grr;
	char c_tmp[32];

	char cmdToSend[32];

	sprintf(cmdToSend, "%s\r\n", cmdToSend_param);

	InsertListItem(data->panel_pilotage, P_PILOT_LS_RS232_ID, -1, cmdToSend, 0);  	

	// Lecture de l'adresse du pousse seringue utilise par la commande cmdToSend
	c_tmp[0]=cmdToSend[0];
	c_tmp[1]=cmdToSend[1];
	c_tmp[2]=NULL;
	
	for(i=0;i<data->nb_PS;i++)
	{
		if(atoi(c_tmp)==data->adresse[i]) adr_valid=1;
	}
	
	// L'adresse utilise par la commande n'est pas valide
	if (adr_valid != 1) return;
	
	
	if(ComWrt(data->com_port,cmdToSend,strlen(cmdToSend)) != strlen(cmdToSend))
	{
		MessagePopup("Attention", "Erreur de transmission");
	}

#ifdef DEBUG_RS232
		// Raffraichissement de l'affichage de la liste 
		grr=GetNumListItems(data->panel_pilotage,P_PILOT_LS_RS232_ID,&err);
		if (err > 400)
			grr=DeleteListItem(data->panel_pilotage, P_PILOT_LS_RS232_ID, 0, err-400);

		grr=GetNumListItems(data->panel_pilotage,P_PILOT_LS_RS232_ID,&err);
		grr=GetCtrlAttribute (data->panel_pilotage, P_PILOT_LS_RS232_ID, ATTR_VISIBLE_LINES, &nb);
		if (err > nb)
			grr=SetCtrlAttribute (data->panel_pilotage, P_PILOT_LS_RS232_ID, ATTR_FIRST_VISIBLE_LINE, err-nb); // scrolling "auto"

		ProcessDrawEvents ();
#endif  

		// Lecture de la rponse du pousse seringue
		// Boucle d'attente d'ETX, gestion du timeout
		i=0;
		do 
		{
			err = ComRdByte(data->com_port);
			// Timeout en lecture
			if(err == -99)
			{
				MessagePopup("Attention", "Pas de rponse d'un des pousses seringues. Vrifiez qu'ils soient mis sous tension.");			
				return;
			}
			c_tmp[i]=err;
			i++;
		}
		while(c_tmp[i-1] != 3);
		

#ifdef DEBUG_RS232 
		c_tmp[i] = 0;
		
		InsertListItem (data->panel_pilotage, P_PILOT_LS_RS232_ID, -1, c_tmp, 0); 
		GetNumListItems(data->panel_pilotage,P_PILOT_LS_RS232_ID,&err);
		if (err > 400)
			DeleteListItem(data->panel_pilotage, P_PILOT_LS_RS232_ID, 0, err-400);

		GetNumListItems(data->panel_pilotage,P_PILOT_LS_RS232_ID,&err);
		GetCtrlAttribute (data->panel_pilotage, P_PILOT_LS_RS232_ID, ATTR_VISIBLE_LINES, &nb);
		if (err > nb)
			SetCtrlAttribute (data->panel_pilotage, P_PILOT_LS_RS232_ID, ATTR_FIRST_VISIBLE_LINE, err-nb); // scrolling "auto"

		ProcessDrawEvents ();
#endif
}
