#include "toolbox.h"
#include <utility.h>
/*
Fichier gnr automatiquement pour l'instrument Regul Acoustic@
*/

#include <ansi_c.h>
#include <userint.h>
#include <rs232.h> 

#include "plugin.h"
#include "UIR_Regul_Acoustic.h"
#include "Regul_Acoustic.h"

PInstrument Regul_Acoustic =
{
NULL

,NULL
,NULL
,NULL
,NULL
,NULL

// Fonctions obligatoires
,Regul_Acoustic_nouveau
,Regul_Acoustic_liberer
,Regul_Acoustic_configure
,Regul_Acoustic_get_config
,Regul_Acoustic_get_nom
,Regul_Acoustic_set_nom

// Stop, start, init, ect..
,Regul_Acoustic_stop
,Regul_Acoustic_start
,Regul_Acoustic_init
,NULL
,Regul_Acoustic_execute
,Regul_Acoustic_piloter

// Mesures 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

// Calculs 1D
,NULL
,NULL
,NULL
,NULL

// Mesures 1D
,Regul_Acoustic_nb_voies_1D
,Regul_Acoustic_get_nom_voie_1D
,Regul_Acoustic_mesure_1D
,Regul_Acoustic_numero_1D

// Calculs 2D
,NULL
,NULL
,NULL
,NULL
,NULL
,NULL

};

//-----------------------------------------------------------------------
// Ajout d'un instrument, remplit la structure PInstrument.
//
// Fonction appelle lors de l'ajout d'un instrument (bouton Ajouter)
// et lors du chargement d'une configuration.
//-----------------------------------------------------------------------
static void Regul_Acoustic_nouveau( PInstrument** instrument )
{
	
	*instrument = (PInstrument*)malloc(sizeof(PInstrument));
	memcpy(*instrument, &Regul_Acoustic, sizeof(PInstrument));
	(*instrument)->structure = malloc(sizeof(infos_Regul_Acoustic));

	sprintf(((infos_Regul_Acoustic*)(*instrument)->structure)->nom_instrument,"Regul Acoustic");
	
	((infos_Regul_Acoustic*)(*instrument)->structure)->panel_pilotage=0;
	((infos_Regul_Acoustic*)(*instrument)->structure)->consigne = 20;
	((infos_Regul_Acoustic*)(*instrument)->structure)->com_port = 1;

	return;
}

//-----------------------------------------------------------------------
// Libre la mmoire alloue  l'instrument.
//
// Appelle  la fin du programme.
//
// Jamais de malloc sans son free !
//-----------------------------------------------------------------------
static void Regul_Acoustic_liberer( PInstrument* instrument )
{
	infos_Regul_Acoustic* data = instrument->structure;

	DiscardPanel(data->panel_pilotage);

	free(instrument->structure);
	free(instrument);

	return;
}

//-----------------------------------------------------------------------
// Copie le nom de l'instrument dans char* nom
//-----------------------------------------------------------------------
static void Regul_Acoustic_get_nom( PInstrument* instrument, char* nom )
{
	infos_Regul_Acoustic* data = instrument->structure;
	
	memcpy(nom, data->nom_instrument, TAILLE_NOM_INSTRU-1);
	return;
}

//-----------------------------------------------------------------------
// Fixe le nom de l'instrument  char* nom
//-----------------------------------------------------------------------
static void Regul_Acoustic_set_nom( PInstrument* instrument, char* nom )
{
	infos_Regul_Acoustic* data = instrument->structure;
	
	memcpy( data->nom_instrument, nom, TAILLE_NOM_INSTRU-1);
	data->nom_instrument[TAILLE_NOM_INSTRU-1] = NULL;
	return;
}

//-----------------------------------------------------------------------
// Appelle la fentre de configuration de l'instrument
//
// On doit fixer les valeurs des controles avant l'apparition de la
// fenetre (cf. commentaires dans le code)
//-----------------------------------------------------------------------
static void Regul_Acoustic_configure( PInstrument* instrument )
{
	infos_Regul_Acoustic* data = instrument->structure;

	int panel;
	if ((panel = LoadPanel (0, "UIR_Regul_Acoustic.uir", P_INSTR)) < 0)
		return ;
	
	// On fixe les valeurs des controles aux valeures courantes.
	
	SetCtrlVal(panel, P_INSTR_RG_COM_ID, data->com_port);
	SetCtrlVal(panel, P_INSTR_NUM_CONSIGNE_ID, data->consigne);
	
	SetCtrlAttribute (panel, P_INSTR_CB_OK, ATTR_CALLBACK_DATA, data);
	DisplayPanel (panel); // Affiche la fentre
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie les paramtres pour sauvegarder la configuration de l'instrument.
//
// !! Ne pas toucher au membre 'dont_remove_me' de la structure      !!
// !! Il sert  diffrencier la partie  sauvegarder de la structure !!
//-----------------------------------------------------------------------
static void Regul_Acoustic_get_config(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_Regul_Acoustic* data = instrument->structure;

	*p_strct = (void*)data;
	
	*p_member = (void*)&(data->dont_remove_me);
	
	*p_bool = 0;
	
	return;
}

//-----------------------------------------------------------------------
// Arrte l'acquisition des donnes de l'instrument.
//
// Cette fonction est appelle :
//  -> A la fin d'une acquisition.
//  -> Lors de l'initialisation si un instrument renvoie
//    une erreur, alors tous sont stopps.
//-----------------------------------------------------------------------
static void Regul_Acoustic_stop( PInstrument* instrument )
{
	infos_Regul_Acoustic* data = instrument->structure;

	HidePanel(data->panel_pilotage);
	InstallComCallback (data->com_port, 0, 0, 0, Regul_Acoustic_com_int, NULL);

	return;
}

//-----------------------------------------------------------------------
// Dmarre l'acquisition des donnes de l'instrument
//
// Cette fonction est appelle :
//  -> Au dmarrage d'une acquisition (bouton Acquisition)
//  -> Au redmararge d'une acq. aprs une pause.
//-----------------------------------------------------------------------
static int Regul_Acoustic_start( PInstrument* instrument )
{
	infos_Regul_Acoustic* data = instrument->structure;

	// Installation d'une interruption  la rception de 't'
	FlushInQ(data->com_port);
	InstallComCallback (data->com_port, 0x0002 ,0,'t', Regul_Acoustic_com_int, (void *)data);

	return 0;
}



//-----------------------------------------------------------------------
// Initialise l'instrument pour une acquisition
//
// Cette fonction est appelle au dmarrage d'une acquisition (bouton Acquisition)
//-----------------------------------------------------------------------
static int Regul_Acoustic_init( PInstrument* instrument )
{
	infos_Regul_Acoustic* data = instrument->structure;
	int	err, i;

	// Charge le panel de pilotage
	if ( data->panel_pilotage == 0)
		if ( ( data->panel_pilotage = LoadPanel (0, "UIR_Regul_Acoustic.uir", P_PILOT)) < 0)
			return -1;

	// Ouverture du port COM RS232
	err=OpenComConfig (data->com_port, NULL, 9600, 0, 8, 1, 512, 512);
	if(err < 0)
	{
		MessagePopup ("Attention", "Impossible d'ouvrir le port demand");
		return err;
	}
	SetXMode(data->com_port, 0);	// Dsactivation du controle de flux logiciel
	SetCTSMode(data->com_port, 0);	// Dsactivation du controle de flux matriel
	SetComTime(data->com_port, 5);	// Activation du timeout en lecture/criture

	for (i=0; i<NOMBRE_VOIES_MESURE1D; i++)
	{
		data->zone_memoire_mes1D[i] = NotANumber();
		data->numero_mes1D[i] = 0;
		
	}
	
	write_consigne_rs232(data);
	
	return 0;
}

//-----------------------------------------------------------------------
// Fonction d'interruption sur le port
//-----------------------------------------------------------------------
void CVICALLBACK Regul_Acoustic_com_int(int COMPort, int eventMask, void *CallbackData)
{
	infos_Regul_Acoustic* data = (infos_Regul_Acoustic *) CallbackData;
	
	int err,nboctets_file,numoctet=0;
	int i;
	char buffer[50];

	if (!(eventMask & 0x0002))
		return;

	if (data==NULL) //Au cas o l'arrt de l'acquisition libre le pointeur et o il y a encore un interruption
		return ;

	// Rcuprer le nombre d'octets dans la file
	nboctets_file = GetInQLen (data->com_port);
	if (nboctets_file ==0) // l 'interruption qui ne sert  rien.
		Delay(0.1);
	nboctets_file = GetInQLen (data->com_port);
	if (nboctets_file ==0) // l 'interruption qui ne sert VRAIMENT  rien.
		return;			// on sort
	
	// Lire le 't'
	do {
		err=ComRdByte (data->com_port);
	} while (err != 't');

	// Lecture de la temprature en C
	err=ComRdTerm (data->com_port, buffer, 50, '\n');
	buffer[err]=0;

	// Rcupration des rsultats de mesure
	data->zone_memoire_mes1D[0] = (double) atof(buffer);
	
	// Incrmentation du nombre de mesures.
	data->numero_mes1D[0]++;
	
}


//-----------------------------------------------------------------------
// Execute une commande sur l'instrument.
//
// Permet la commande de la consigne par le squenceur.
// Pour s'assurer d'un bon fonctionnement respecter la syntaxe suivante :
// "consigne=xx.xx" ou x reprsente un entier de 0  9.
//
//-----------------------------------------------------------------------
static void Regul_Acoustic_execute( PInstrument* instrument, char* chaine)
{
	infos_Regul_Acoustic *data = instrument->structure;
	double d_consigne;	
	
	if(sscanf(chaine,"consigne=%lf",&d_consigne)==1)
	{
		// Envoi de la consigne au rgulateur
		data->consigne=d_consigne;
		write_consigne_rs232(data);
		
		// Mise  jour de l'affichage
		SetCtrlVal(data->panel_pilotage, P_PILOT_NUM_CONSIGNE_ID, d_consigne);
	}

	return;
}

//-----------------------------------------------------------------------
// Active le pilotage de l'instrument.
//
// <TODO>
//-----------------------------------------------------------------------
static void Regul_Acoustic_piloter( PInstrument* instrument )
{

	infos_Regul_Acoustic *data = instrument->structure;
	
	SetCtrlVal(data->panel_pilotage, P_PILOT_NUM_CONSIGNE_ID, data->consigne);
	
	SetCtrlAttribute(data->panel_pilotage, P_PILOT_CB_VALIDATE_ID, ATTR_CALLBACK_DATA, data);
	
	DisplayPanel(data->panel_pilotage);

	return;
}



//-----------------------------------------------------------------------
// Renvoie le nombre de voies de l'instrument.
//-----------------------------------------------------------------------
static int Regul_Acoustic_nb_voies_1D( PInstrument* instrument )
{
	return NOMBRE_VOIES_MESURE1D;
}

//-----------------------------------------------------------------------
// Renvoie le nom de la voie 'int voie' de l'instrument.
//-----------------------------------------------------------------------
static void Regul_Acoustic_get_nom_voie_1D( PInstrument* instrument, int voie, char* string )
{

	if (voie == 2)
		sprintf(string, "Consigne");
	else
		sprintf(string,"Regulation");
	
	return;
}

//-----------------------------------------------------------------------
// Renvoie la mesure courante de la voie 'int voie' de l'instrument.
//-----------------------------------------------------------------------
static double Regul_Acoustic_mesure_1D( PInstrument* instrument, int voie )
{
	infos_Regul_Acoustic* data = instrument->structure;

	return data->zone_memoire_mes1D[voie-1];
}

//-----------------------------------------------------------------------
// Renvoie le numro de la mesure courante de la voie 'int voie' de
// l'instrument.
//-----------------------------------------------------------------------
static unsigned int Regul_Acoustic_numero_1D( PInstrument* instrument, int voie )
{
	infos_Regul_Acoustic* data = instrument->structure;

	return data->numero_mes1D[voie-1];
}



/*-----------------------------------------------------------------*/
/*              CALLBACKS INTERFACE UTILISATEUR
/*-----------------------------------------------------------------*/

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Ok' du panneau de configuration de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Regul_Acoustic_config_ok(int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int i_tmp;
	double d_tmp;

	infos_Regul_Acoustic* data = (infos_Regul_Acoustic*)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:

			// Rcupre le numro du port COM
			GetCtrlVal(panel, P_INSTR_RG_COM_ID, &i_tmp);
			data->com_port = i_tmp;
			
			// Rcupre la consigne
			GetCtrlVal(panel, P_INSTR_NUM_CONSIGNE_ID, &d_tmp);
			data->consigne = d_tmp;

			DiscardPanel(panel);
			
			break;
		}
	return 0;
}


//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Cacher' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Regul_Acoustic_hide(int panel, int control, int event,
	void *callbackData, int eventData1, int eventData2)
{

 	switch (event)
	{
		case EVENT_COMMIT:
			HidePanel(panel);
			break;
	}
	
	return 0;
}

//-----------------------------------------------------------------------
// CALLBACK pour le bouton 'Envoyer' du panneau de pilotage de
// l'instrument.
//-----------------------------------------------------------------------
int CVICALLBACK CB_Regul_Acoustic_pilot_VALIDATE (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	infos_Regul_Acoustic* data = (infos_Regul_Acoustic*)callbackData;	
	
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal(data->panel_pilotage, P_PILOT_NUM_CONSIGNE_ID, &data->consigne);
			write_consigne_rs232(data);
			break;
		}
	return 0;
}

//-----------------------------------------------------------------------
//
// Fonction d'envoi de la consigne sur la RS232. Gestion du protocole
// de communication avec la carte PIC.
//
// La fonction "gets" du PIC attend un CR (13 en dcimal, \r) pour indiquer
// la fin de la lecture du buffer de rception.
//
//-----------------------------------------------------------------------
void write_consigne_rs232(infos_Regul_Acoustic * data)
{
	int err=0;
	char consigne_ASCII[10];

	// Envoi de "c"
	if(ComWrtByte(data->com_port, 'c') != 1)
	{
		MessagePopup("Attention", "Erreur de transmission");
	}

	// Attente de "y"
	do
	{
		err=ComRdByte(data->com_port);
	}
	while((char)err != 'y');
	
	// Une attente sinon le PIC n'a pas le temps de ragir ?
	Delay(0.1);
	
	// Formattage de la consigne pour l'envoi vers le PIC
	sprintf(consigne_ASCII, "%2.2f%c%c", data->consigne,13,13);
	
	// Envoi de la consigne
	if(ComWrt(data->com_port, consigne_ASCII, strlen(consigne_ASCII)) != strlen(consigne_ASCII))
	{
		MessagePopup("Attention", "Erreur de transmission");
	}
	
	if (NOMBRE_VOIES_MESURE1D > 1)
	{
		data->zone_memoire_mes1D[1] = data->consigne;
		data->numero_mes1D[1]++;
	}
}


