#include "toolbox.h"
#include <formatio.h>
#include <gpib.h>
#include <userint.h>
#include <ansi_c.h>
#include "plugin.h"
#include "enceinte_thermique.h"
#include <utility.h> // pour NULL (au moins) :)
#include "UIR_enceinte_thermique.h"
#define EOT 0x04
#define STX 0x02
#define ETX 0x03
#define ENQ 0x05
#define CR  0x0D
#define LF  0x0A

PInstrument enc_ther =
{
	NULL, // Rempli par 'nouveau'

	NULL, // rempli aprs nouveau (par liste.c)
	NULL,
	NULL,
	NULL,
	NULL,

	enceinte_nouveau,
	enceinte_liberer,
	enceinte_configure,
	enceinte_getconfig,
	enceinte_get_nom,
	enceinte_set_nom,
	NULL,
	NULL,
	enceinte_init,

	enceinte_status,
	enceinte_execute,
	enceinte_piloter,

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};
///////////////////////////////PARTIE GRAPHIQUE/////////////////////////////////////////////
static void enceinte_nouveau(PInstrument **enceinte)
{
	*enceinte = (PInstrument *)malloc(sizeof(PInstrument));
	memcpy (*enceinte, &enc_ther, sizeof(PInstrument));
	(*enceinte)->structure = malloc(sizeof(info_enceinte));
	((info_enceinte *)(*enceinte)->structure)->adresse_GPIB = 3;
	((info_enceinte *)(*enceinte)->structure)->Temp = 0;
	((info_enceinte *)(*enceinte)->structure)->panel_pilotage = -1;
	sprintf(((info_enceinte *)(*enceinte)->structure)->nom_pilot, "Enceinte thermique");
}

static void enceinte_liberer(PInstrument *enceinte)
{
	DiscardPanel(((info_enceinte *)enceinte->structure)->panel_pilotage);
	free(enceinte->structure);
	free(enceinte);
}

static void enceinte_configure(PInstrument *enceinte)
{
	info_enceinte *data_enceinte = enceinte->structure;
	int panel;
	
	if ((panel = LoadPanel (0, "UIR_enceinte_thermique.uir", P_config)) < 0)
		return ;
		
	SetCtrlVal (panel, P_config_NUMERIC, data_enceinte->adresse_GPIB);
	SetCtrlAttribute (panel, P_config_BUTTON_OK, ATTR_CALLBACK_DATA, data_enceinte);
	
	DisplayPanel (panel); // Affiche la fentre
}

static void enceinte_getconfig (PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{
	info_enceinte* data = instrument->structure;
	
	*p_strct = (void*)data;
	*p_member = (void*)&(data->zone_memoire);
	
	*p_bool = 0;
}

static void enceinte_get_nom (PInstrument *enceinte, char *nom_pilot)
{
	memcpy(nom_pilot, ((info_enceinte *)enceinte->structure)->nom_pilot, TAILLE_NOM_INSTRU-1);
	
}

static void enceinte_set_nom (PInstrument *enceinte, char *nom_pilot)
{
	memcpy(((info_enceinte *)enceinte->structure)->nom_pilot, nom_pilot, TAILLE_NOM_INSTRU-1);
	((info_enceinte *)enceinte->structure)->nom_pilot[TAILLE_NOM_INSTRU-1] = NULL;
}

static int enceinte_init (PInstrument *enceinte)
{	  
	int id_enceinte=0;
	short shorttmp=0;
	int panel_pilotage=0;
	
	info_enceinte *data_enceinte = enceinte->structure; 
	
	data_enceinte->zone_memoire[0]=NotANumber();
	data_enceinte->zone_memoire[1]=NotANumber();
	 
	id_enceinte = ibdev (0, data_enceinte->adresse_GPIB, NO_SAD, T10s, 1, (4<<8)+0xa);
	ibclr (id_enceinte);		
	ibln (id_enceinte, data_enceinte->adresse_GPIB, NO_SAD, &shorttmp); 
	if (shorttmp== 0) 
		{
		MessagePopup ("Attention", "Il faudrait allumer l'enceinte");
		return -1;
		}
		
	data_enceinte->id_enceinte = id_enceinte;
	
	// Dtection de l'appareil
	// Configuration
	// ...
	// Chargement du panel pour le pilotage manuel (et mmorisation du panel_handle dans la structure)
	
	if ( ((info_enceinte *)enceinte->structure)->panel_pilotage < 0)
		if ( ( panel_pilotage = LoadPanel (0, "UIR_enceinte_thermique.uir", P_PILOT)) < 0)
			return -1;
		else
			((info_enceinte *)enceinte->structure)->panel_pilotage = panel_pilotage;
	
	SetCtrlVal (panel_pilotage, P_PILOT_NUMERIC, data_enceinte->Temp);
	SetCtrlAttribute (panel_pilotage, P_PILOT_BUTTON_2, ATTR_CALLBACK_DATA, data_enceinte); 
 
	Debut_enceinte(data_enceinte->id_enceinte); 

	return 0; // ngatif = erreur
}

static int enceinte_status (PInstrument *enceinte) // La fonction devrait, dans l'idal, ne pas tre blocante (pour le bon fonctionnement du squenceur)
{
	return STATUS_PRET;
	// return STATUS_OQP;
}

static void enceinte_execute (PInstrument *enceinte, char *chaine) // La fonction devrait, dans l'idal, ne pas tre blocante (pour le bon fonctionnement du squenceur)
{
	// Interprtation de 'chaine'
	// Envoi vers l'appareil
	// Mise  jour du panel pour le pilotage manuel
	
	float Valeur_Temperature;
	info_enceinte *structu_enceinte=(info_enceinte *)enceinte->structure;
   
	//strcmp(chaine, "T=XY");
	if (sscanf(chaine,"T=%f",&Valeur_Temperature) == 1)
	{
		SetCtrlVal ( structu_enceinte->panel_pilotage, P_PILOT_NUMERIC, Valeur_Temperature);
		structu_enceinte->Temp = Valeur_Temperature; 
		Temperature(Valeur_Temperature,structu_enceinte->id_enceinte);
	}
	
	return;
}

static void enceinte_piloter(PInstrument *enceinte)
{
	info_enceinte *structu_enceinte = enceinte->structure;
	DisplayPanel(structu_enceinte->panel_pilotage);
}

////////////////////////////LES CALLBACKS/////////////////////////////////////////////////
int CVICALLBACK OK_conf (int panel, int control, int event,
		  void *callbackData, int eventData1, int eventData2)
{
	int Valeur_adresse_GPIB=0;
	
	info_enceinte *structu_enceinte;
	structu_enceinte = (info_enceinte *)callbackData;
	
	switch (event)						 
		{
		case EVENT_COMMIT:
		
			GetCtrlVal (panel, P_config_NUMERIC, &Valeur_adresse_GPIB);
			structu_enceinte->adresse_GPIB =Valeur_adresse_GPIB;
		
			DiscardPanel(panel);
			break;
		}
	return 0;
}

int CVICALLBACK OK_pilote (int panel_pilotage, int control, int event,
		  void *callbackData, int eventData1, int eventData2)
{
	float Valeur_Temperature=0;
	
	info_enceinte *structu_enceinte;
	structu_enceinte = (info_enceinte *)callbackData;

	
	switch (event)
		{
		case EVENT_COMMIT:
			
			GetCtrlVal (panel_pilotage, P_PILOT_NUMERIC, &Valeur_Temperature);
			structu_enceinte->Temp = Valeur_Temperature;
			Temperature(Valeur_Temperature,structu_enceinte->id_enceinte);
			
			break;						  
		}
	return 0;
}						 

int CVICALLBACK Cacher (int panel, int control, int event,
				void *callbackData, int eventData1, int eventData2)
{		

	info_enceinte *structu_enceinte;
	structu_enceinte = (info_enceinte*)callbackData;
	
	switch (event)
		{
		case EVENT_COMMIT:
			
			HidePanel(panel);
			
			break;
		}
	return 0;
}

//////////////////////////PILOTAGE DE L'ENCEINTE////////////////////////////////////// 
static void Debut_enceinte(int id_enceinte)
{	
	char *chaine="MIP RS=26",chainetotale[30];
	int err=0;
	
	sprintf(chainetotale,"%s%c",chaine,CR);
	err=ibwrt(id_enceinte,chainetotale,strlen(chainetotale)); 

} 

static void Ecrit_Eurotherm(char *adresse, char *mnemo,int id_enceinte)
{   
	int long_mnemo,bcc,i;
	char write_buffer[30];
	
	bcc=0x03;
	long_mnemo = StringLength (mnemo);

	for (i=0;i<long_mnemo;i++)
	  {bcc=mnemo[i]^bcc;}

	sprintf (write_buffer,"%c%s%c%s%c%c%c",EOT,adresse,STX,mnemo,ETX,bcc,CR);

	ibwrt (id_enceinte, write_buffer, strlen(write_buffer));
}

static void Temperature(float Temper,int id_enceinte)
{
	char adrr[5], temp[50]; 
	temp[49]=0;

	sprintf(temp,"SL%05.1f",Temper);
	sprintf(adrr,"0011"); 
	Ecrit_Eurotherm(adrr , temp,id_enceinte);
	
}	

static float lecture (int id_enceinte)
{
	char write_buffer[30],Trame[30],n;
	float lec;
	
    sprintf(write_buffer,"%c%s%s%c%c",EOT,"0011","PV",ENQ,CR);

    ibwrt (id_enceinte, write_buffer, strlen(write_buffer));   
	ibrd (id_enceinte, Trame , 30);
	
	for(n=0;n<30;n++)
		if(Trame[n]==ETX) Trame[n]=0;
		
	sscanf(Trame+1, "PV%f\0",&lec);
	return lec;		
}
