#include "toolbox.h"
#include <visa.h>
#include <utility.h>
#include "hp5313xa.h"
#include <userint.h>
#include <ansi_c.h>
#include <gpib.h>
#include "globales.h" 
#include "freq2voies.h"
#include "UIR_freq2voies.h"
#include "plugin.h"

PInstrument freq2v =
{
	NULL, // Rempli par 'nouveau'

	NULL, // rempli aprs nouveau (par liste.c)
	NULL,
	NULL,
	NULL,
	NULL,

	frequencemetre_2voies_nouveau,
	frequencemetre_2voies_liberer,
	frequencemetre_2voies_configure,
	frequencemetre_2voies_getconfig,
	frequencemetre_2voies_get_nom,
	frequencemetre_2voies_set_nom,
	arreter_interruption_freq_2voies,
	miseenplace_int_freq_2voies,
	initialisation_freq_2voies_int,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	
	frequencemetre_2voies_nb_voie,
	frequencemetre_2voies_get_nom_voie,
	frequencemetre_2voies_mesure,
	frequencemetre_2voies_numero,
	
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

char Type_Appareil[255];

int initialisation_freq_2voies_int(PInstrument *frequencemetre)
{
	infos_freq_2voies *data_voie = frequencemetre->structure;
	char gpib_canal[20], chaine[255], tmp[255], chaine_digits[255];
	int err=0,handle=0,id_cont=0;
	short err_short=0;

	// RAZ du watchdog
	data_voie->watchdog = 0;
	
	// Initialisation Zone Mmoire.
	data_voie->zone_memoire[0]=NotANumber();
	data_voie->zone_memoire[1]=NotANumber();


	//Initialisation Numro Mesure.
	data_voie->numero_mesure[0] = 0;
	data_voie->numero_mesure[1] = 0;

	err = ibfind("GPIB0");
	if (err & ERR)
	{
		MessagePopup ("Problme de carte", "Carte GPIB non dtecte");
		return -1;
	}

	// detection frquencemtre
	id_cont = ibdev (0, data_voie->adresse_gpib, NO_SAD, T10s, 1, 1);
	ibclr (id_cont);		// Erreur CVI si branch mais pas allum !?
	ibln (id_cont, data_voie->adresse_gpib, NO_SAD, &err_short); 

	if (err_short == 0) 
		{
		sprintf(chaine,"Il faudrait penser  brancher le compteur  l'adresse %d !!!",data_voie->adresse_gpib);
		MessagePopup ("PAS BON", chaine);
		return -1;
		}
	
	ibecr(id_cont, "*IDN?");
	ibrd(id_cont, chaine, 250);
	if (strstr (chaine, "HEWLETT-PACKARD,5313") == NULL)
	{
		MessagePopup ("Erreur", "Modle non support par ce module !");
		return -1;
	}
	
	sprintf(tmp,"%d",data_voie->digits);
	strcpy(chaine_digits,":FREQ:ARM:STOP:DIG ");
	strcat (chaine_digits, tmp);
	sprintf(gpib_canal, "GPIB::%d", data_voie->adresse_gpib);
	if ((err = hp5313xa_init (gpib_canal, VI_OFF, VI_ON, 1, &handle))!=0) 
		analyse_msg_VI("init",err);
	else if ((err = hp5313xa_reset (handle))!=0) // reset
		analyse_msg_freq("reset",err);
	else if ((err = hp5313xa_writeInstrData (handle, "*CLS"))!=0)   // clear event register and error queue
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, "*SRE 0"))!=0) // clear service request enable register
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, "*ESE 0"))!=0) // clear event status enable register
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":FUNC 'FREQ 1'"))!=0) // utilis pour mesurer la frquence, voie 1
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":FREQ:ARM:STAR:SOUR IMM"))!=0) // dclenchement immdiat
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":FREQ:ARM:STOP:SOUR DIG"))!=0) // Arrt sur un nombre de Digits
		analyse_msg_freq("writeInstrData",err);	
	else if ((err = hp5313xa_writeInstrData (handle, chaine_digits))!=0) // Spcification du nombre de Digits
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":TRIG:COUN:AUTO OFF"))!=0) // pour que :init:imm dmarre une mesure directement
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, "FORM:DATA ASCII"))!=0) // on renvoit des nombres rels.
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_inputCond (handle, 1, data_voie->couplage, data_voie->impedance, data_voie->attenuation, data_voie->filtre))!=0)
		analyse_msg_freq("inputCond",err);
	else if ((err = hp5313xa_trigSens (handle,  1, VI_OFF, 50, data_voie->attenuation, VI_ON, 0))!=0)
		analyse_msg_freq("trigSens",err);
	else if ((err = hp5313xa_inputCond (handle, 2, data_voie->couplage, data_voie->impedance, data_voie->attenuation, data_voie->filtre))!=0)
		analyse_msg_freq("inputCond",err);
	else if ((err = hp5313xa_trigSens (handle,  2, VI_OFF, 50, data_voie->attenuation, VI_ON, 0))!=0)
		analyse_msg_freq("trigSens",err);
	else if (( err = viSetAttribute (handle, VI_ATTR_TMO_VALUE, 2500)) !=0)
		analyse_msg_VI("Timeout",err);
	hp5313xa_close (handle);

	
	// Mise  jour des donnes de la structure.
	data_voie->identificateur = id_cont;
	data_voie->stop_interruption = RQS; 
	
	return err;
}

/* Fonction de mise en place de l'interruption sur le freq 2 voies.
Renvoit CMPL (0x0100) en cas de succs.*/
int miseenplace_int_freq_2voies(PInstrument *frequencemetre)
{
	infos_freq_2voies *data_voie = frequencemetre->structure;
	int err=0;
	char chaine[255];				  
	unsigned short addr_GPIB=0;
	//initialisation de la section de code critique
	//InitializeCriticalSection(&criticalSection);

	//err = ibclr (data_voie->identificateur); 
	
	data_voie->panel_wdog = LoadPanel(0,"UIR_freq2voies.uir",P_WD);
	
	// Mise en place du timer / watchdog
	SetCtrlAttribute (data_voie->panel_wdog, P_WD_TIMER, ATTR_CALLBACK_DATA, frequencemetre);

	err=ibecr(data_voie->identificateur,"*ESE 1");
	// Activation de l'interruption  la fin des mesures sur le frquencemtre
	if ((err = ibecr (data_voie->identificateur, "*SRE 32")) & ERR) // Activation
	{
		sprintf(chaine,"Impossible d'initialiser l'interruption\nsur le frquencemtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_voie->identificateur,iberr,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		return err;
	}

	// Nettoyage du registre d'interruptions
	ibecr(data_voie->identificateur,"*ESR?");
	ibrd(data_voie->identificateur,chaine,10);
	
	CmtNewLock(NULL, 0, &data_voie->lock_interruption);
	
	// Mise en place de l'interruption
	err = ibnotify (data_voie->identificateur, data_voie->stop_interruption, CB_interruption_freq_2voies,frequencemetre);
	if (err & ERR)
	{
		sprintf(chaine,"Impossible d'initialiser le frquencemtre\nd'adresse %d",addr_GPIB);
		MessagePopup("Attention !!!",chaine);
		return -1;
	}
	// lancement d'une mesure
	if ((err = ibecr (data_voie->identificateur, ":INIT:IMM;*WAI;*OPC"))& ERR) // dclenchement immdiat de la mesure
	{
		sprintf(chaine,"Impossible de lancer une mesure\nsur le frquencemtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_voie->identificateur,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		ibnotify (data_voie->identificateur, 0, CB_interruption_freq_2voies,data_voie->zone_memoire); //Dsactivation de l'interruption
		return err;
	}
	return err;
}
	

int __stdcall CB_interruption_freq_2voies(int boardOrDevice, int localsta, int localerr, long localcntl, void *callbackData)
{
	infos_freq_2voies *data_voie;
	

	char chaine[256],chaine_voie[255];
	int err;
	
	int lock=-1;

	switch (localsta)
	{
		case ERR:
			return 0;
		default:
	
			data_voie = (infos_freq_2voies *)( ((PInstrument*)callbackData)->structure );

			data_voie->etat_voie = (!(data_voie->etat_voie));
	
			sprintf(chaine, "GPIB0:%i", data_voie->identificateur);
			lock = ((PInstrument*)callbackData)->gestion_lock(chaine);
			
			// On lock
			CmtGetLock(lock);
			// RAZ du watchdog
			data_voie->watchdog = 0;
			CmtReleaseLock(lock);
	
			// Enlever le bit RQS
			ibrsp(boardOrDevice,chaine);

			// Demander la frquence au frquencemtre
			ibecr(boardOrDevice,"FETCH:FREQ?");
			//Lecture de  la chaine
			ibrd (boardOrDevice, chaine, 255);
	
			chaine[ThreadIbcntl()]='\0'; //Ajout du caractere de fin de chaine
						  //"+1.2346666666656E+006."
			err = sscanf (chaine, "%lf", &(data_voie->zone_memoire[!(data_voie->etat_voie)])); //transformation en flottant
    
		    // Incrmentation du nombre de mesures.
		    data_voie->numero_mesure[!(data_voie->etat_voie)]++;
      
			if (err==EOF)		
				data_voie->zone_memoire[!(data_voie->etat_voie)]=NotANumber(); // Debug du sscanf -> Si erreur, envoie 18  la place de la mesure.
	

			Delay(0.5);
			
			// Basculement des voies.
			sprintf(chaine_voie,":FUNC 'FREQ %d'",(data_voie->etat_voie)+1);
			ibecr(boardOrDevice, chaine_voie); // utilis pour mesurer la frquence, voie 1

			// Ca deconne apres, parfois (!)
			// parade trs exotique, apparement efficace
			err = ibecr(boardOrDevice, "*IDN?");
			if (err & ERR)
				Delay(0); // instruction pour breaker
				//return data_voie->stop_interruption;
			else
				err = ibrd(boardOrDevice, chaine, 250);
			// fin de la parade, merci de votre attention
		
			// Nettoyage du registre d'interruptions
			ibecr(boardOrDevice,"*ESR?");
			ibrd(boardOrDevice,chaine,10);

			// Enlever le bit RQS
			ibrsp(boardOrDevice,chaine);

			// Relancement d'une mesure
			ibecr (boardOrDevice, ":INIT:IMM;*OPC"); // dclenchement immdiat
	
			return data_voie->stop_interruption; // Pour ractiver l'interruption
	}
}

static void arreter_interruption_freq_2voies(PInstrument *frequencemetre)
{
	infos_freq_2voies *data_voie = frequencemetre->structure;
	
	if (data_voie->stop_interruption == 0) // dj arrt
		return;
		
	//dsactivation de l'interruption
	data_voie->stop_interruption = 0;
	
	DiscardPanel(data_voie->panel_wdog);

	//ibonl(data_voie->identificateur,0);
}

static void frequencemetre_2voies_nouveau(PInstrument **frequencemetre)
{
	*frequencemetre = (PInstrument *)malloc(sizeof(PInstrument));
	memcpy (*frequencemetre, &freq2v, sizeof(PInstrument));
	(*frequencemetre)->structure = (infos_freq_2voies *)malloc(sizeof(infos_freq_2voies));
	((infos_freq_2voies *)(*frequencemetre)->structure)->adresse_gpib = 3;
	sprintf(((infos_freq_2voies *)(*frequencemetre)->structure)->nom_instru, "Frquencemtre 2 voies");
	
	((infos_freq_2voies *)(*frequencemetre)->structure)->delay_wdog = 3.0;
}

static void frequencemetre_2voies_liberer(PInstrument *instru)
{
	free(instru->structure);
	free(instru);
}

static void frequencemetre_2voies_get_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(nom_instru, ((infos_freq_2voies *)instrument->structure)->nom_instru, TAILLE_NOM_INSTRU-1);
}

static void frequencemetre_2voies_set_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(((infos_freq_2voies *)instrument->structure)->nom_instru, nom_instru, TAILLE_NOM_INSTRU-1);
	((infos_freq_2voies *)instrument->structure)->nom_instru[TAILLE_NOM_INSTRU-1] = NULL;
}

static void frequencemetre_2voies_configure(PInstrument *frequencemetre)
{
	infos_freq_2voies *data_freq = frequencemetre->structure;
	
	int panel;
	if ((panel = LoadPanel (0, "UIR_freq2voies.uir", P_Conf_F_2)) < 0)
		return ;
					   
	SetCtrlVal (panel, P_Conf_F_2_NUM_Conf_Adr_Freq, data_freq->adresse_gpib);
	SetCtrlVal (panel, P_Conf_F_2_RING_Conf_Digit_Freq, data_freq->digits);
	SetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Impedan, data_freq->impedance);
	SetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Filtre, data_freq->filtre);
	SetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Couplag, data_freq->couplage);
	SetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Attenua, data_freq->attenuation);
	SetCtrlVal (panel, P_Conf_F_2_NUM_WD_Delay, data_freq->delay_wdog);

	SetCtrlAttribute (panel, P_Conf_F_2_Conf_OK2V, ATTR_CALLBACK_DATA, data_freq);
	
	DisplayPanel (panel); // Affiche la fentre "CONFIGURATION DE L'APPAREIL" pour un frquencemtre
}

static void frequencemetre_2voies_getconfig(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{
	infos_freq_2voies *data = instrument->structure;
	
	*p_strct = (void*)data;
	*p_size = sizeof(infos_freq_2voies);
	
	*p_bool = 1;
	
}

static int frequencemetre_2voies_nb_voie(PInstrument *frequencemetre)
{
	infos_freq_2voies *data_voie = frequencemetre->structure;
	return data_voie->nombre_voie;
}

static void frequencemetre_2voies_get_nom_voie(PInstrument *frequencemetre, int voie, char *nom_voie)
{
	sprintf(nom_voie, "Voie %i", voie);
}

static double frequencemetre_2voies_mesure(PInstrument *frequencemetre, int voie)
{
	infos_freq_2voies *data_freq = frequencemetre->structure;
	return data_freq->zone_memoire[voie-1];
}

static unsigned int frequencemetre_2voies_numero(PInstrument *frequencemetre, int voie)
{
	infos_freq_2voies *data_freq = frequencemetre->structure;
	return data_freq->numero_mesure[voie-1];
}

static int frequencemetre_2voies_digits(PInstrument *frequencemetre)
{
	infos_freq_2voies *data_freq = frequencemetre->structure;
	return data_freq->digits;
}

int CVICALLBACK Conf_F2V_OK (int panel, int control, int event,void *callbackData, int eventData1, int eventData2)
{

	int Valeurs_Configuration_Adresse_GPIB = 0;
	int Valeurs_Configuration_Digit = 0;
	int Valeurs_Configuration_Impedance = 0;
	int Valeurs_Configuration_Filtre = 0;
	int Valeurs_Configuration_Couplage = 0;
	int Valeurs_Configuration_Attenuation = 0;

	infos_freq_2voies *structu_inst2;

	structu_inst2 = (infos_freq_2voies *)callbackData;

	
	switch (event)
		{
		case EVENT_COMMIT:
		
			GetCtrlVal (panel, P_Conf_F_2_NUM_Conf_Adr_Freq,&Valeurs_Configuration_Adresse_GPIB); // Rcupration de l'adresse GPIB
			GetCtrlVal (panel, P_Conf_F_2_RING_Conf_Digit_Freq,&Valeurs_Configuration_Digit); // Rcupration du nombre de digits
			GetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Impedan, &Valeurs_Configuration_Impedance); // Rcupration de la valeur de l'impdance
			GetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Filtre, &Valeurs_Configuration_Filtre); // Rcupration de l'tat du filtre
			GetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Couplag, &Valeurs_Configuration_Couplage); // Rcupration de la nature du couplage
			GetCtrlVal (panel, P_Conf_F_2_BINARYSW_Conf_Attenua, &Valeurs_Configuration_Attenuation); // Rcupration de la valeur de l'attnuation
			GetCtrlVal (panel, P_Conf_F_2_NUM_WD_Delay         , &(structu_inst2->delay_wdog)); // Rcupration de la valeur du delai du watchdog
			
			structu_inst2->adresse_gpib = Valeurs_Configuration_Adresse_GPIB;
			structu_inst2->digits = Valeurs_Configuration_Digit;
			structu_inst2->impedance = Valeurs_Configuration_Impedance;
			structu_inst2->filtre = Valeurs_Configuration_Filtre;
			structu_inst2->couplage = Valeurs_Configuration_Couplage;
			structu_inst2->attenuation = Valeurs_Configuration_Attenuation;
			structu_inst2->nombre_voie = 2;
			
			DiscardPanel (panel); // Cache la fentre "CONFIGURATION D'UN APPAREIL" pour un frquencemtre
			
			break;
		}
	return 0;
}

int CVICALLBACK CB_freq2voies_watchdog (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	infos_freq_2voies *data_voie = ((PInstrument*)callbackData)->structure;

	char chaine[64];
	
	int lock = -1;
	
	int truc;

	switch (event)
		{
		case EVENT_TIMER_TICK:
		
			(data_voie->watchdog)++;
		
			sprintf(chaine, "GPIB0:%i", data_voie->identificateur);
			lock = ((PInstrument*)callbackData)->gestion_lock(chaine);
			
			// On lock
			CmtGetLock(lock);
			
			truc =  (int)(1.0*(data_voie->delay_wdog));
		
			if( data_voie->watchdog > (int)(1.0*(data_voie->delay_wdog)) )
			{
				//MessagePopup("!!!","Wouaf !");
				
				data_voie->watchdog = 0;
			
				data_voie->etat_voie = (!(data_voie->etat_voie));
			
				// Basculement des voies.
				sprintf(chaine,":FUNC 'FREQ %d'",(data_voie->etat_voie)+1);
				ibecr(data_voie->identificateur, chaine); // utilis pour mesurer la frquence, voie 1

				// Nettoyage du registre d'interruptions
				ibecr(data_voie->identificateur,"*ESR?");
				ibrd(data_voie->identificateur,chaine,10);

				// Enlever le bit RQS
				ibrsp(data_voie->identificateur,chaine);

				// Relancement d'une mesure
				ibecr (data_voie->identificateur, ":INIT:IMM;*OPC"); // dclenchement immdiat
			}

			CmtReleaseLock(lock);
	
			break;
		}
	return 0;
}
