#include "toolbox.h"
#include <visa.h>
#include <utility.h>
#include "hp5313xa.h"
#include <userint.h>
#include <ansi_c.h>
#include <gpib.h>
#include "globales.h" 
#include "frequencemetre_SW.h"
#include "UIR_frequencemetre_SW.h"

#include "plugin.h"

PInstrument freq_sw =
{
	NULL, // Rempli par 'nouveau'

	NULL, // rempli aprs nouveau (par liste.c)
	NULL,
	NULL,
	NULL,
	NULL,

	frequencemetre_SW_nouveau,
	frequencemetre_SW_liberer,
	frequencemetre_SW_configure,
	frequencemetre_SW_getconfig,
	frequencemetre_SW_get_nom,
	frequencemetre_SW_set_nom,
	arreter_interruption_freq_switch,
	miseenplace_int_freq_switch,
	initialisation_freq_switch_int,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	
	frequencemetre_SW_nb_voie,
	frequencemetre_SW_get_nom_voie,
	frequencemetre_SW_mesure,
	frequencemetre_SW_numero,
	
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

int initialisation_freq_switch_int(PInstrument *frequencemetre)
{
	infos_frsw *data_switch = frequencemetre->structure;
	char gpib_canal[20], chaine[255], tmp[255], chaine_digits[255];
	int err=0,handle=0,id_cont=0,val_port_para=0;
	short err_short=0;


	// Initialisation Zone Mmoire.
	data_switch->zone_memoire[0]=NotANumber();
	data_switch->zone_memoire[1]=NotANumber();


	//Initialisation Numro Mesure.
	data_switch->numero_mesure[0] = 0;
	data_switch->numero_mesure[1] = 0;

	// detection frquencemtre
	id_cont = ibdev (0, data_switch->adresse_gpib, NO_SAD, T10s, 1, 1);
	ibclr (id_cont);		// Erreur CVI si branch mais pas allum !?
	ibln (id_cont, data_switch->adresse_gpib, NO_SAD, &err_short); 

	if (err_short == 0) 
		{
		sprintf(chaine,"Il faudrait penser  brancher le compteur  l'adresse %d !!!",data_switch->adresse_gpib);
		MessagePopup ("PAS BON", chaine);
		return -1;
		}
	
	ibecr(id_cont, "*IDN?");
	ibrd(id_cont, chaine, 250);
	if (strstr (chaine, "HEWLETT-PACKARD,5313") == NULL)
	{
		MessagePopup ("Erreur", "Modle non support par ce module !");
		return -1;
	}

	sprintf(tmp,"%d",data_switch->digits);
	strcpy(chaine_digits,":FREQ:ARM:STOP:DIG ");
	strcat (chaine_digits, tmp);
	sprintf(gpib_canal, "GPIB::%d", data_switch->adresse_gpib);
	if ((err = hp5313xa_init (gpib_canal, VI_OFF, VI_ON, 1, &handle))!=0) 
		analyse_msg_VI("init",err);
	else if ((err = hp5313xa_reset (handle))!=0) // reset
		analyse_msg_freq("reset",err);
	else if ((err = hp5313xa_writeInstrData (handle, "*CLS"))!=0)   // clear event register and error queue
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, "*SRE 0"))!=0) // clear service request enable register
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, "*ESE 0"))!=0) // clear event status enable register
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":FUNC 'FREQ 1'"))!=0) // utilis pour mesurer la frquence, voie 1
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":FREQ:ARM:STAR:SOUR IMM"))!=0) // dclenchement immdiat
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":FREQ:ARM:STOP:SOUR DIG"))!=0) // Arrt sur un nombre de Digits
		analyse_msg_freq("writeInstrData",err);	
	else if ((err = hp5313xa_writeInstrData (handle, chaine_digits))!=0) // Spcification du nombre de Digits
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, ":TRIG:COUN:AUTO OFF"))!=0) // pour que :init:imm dmarre une mesure directement
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_writeInstrData (handle, "FORM:DATA ASCII"))!=0) // on renvoit des nombres rels.
		analyse_msg_freq("writeInstrData",err);
	else if ((err = hp5313xa_inputCond (handle, 1, data_switch->couplage, data_switch->impedance, data_switch->attenuation, data_switch->filtre))!=0)
		analyse_msg_freq("inputCond",err);
	else if ((err = hp5313xa_trigSens (handle,  1, VI_OFF, 50, data_switch->attenuation, VI_ON, 0))!=0)
		analyse_msg_freq("trigSens",err);
	else if ((err = hp5313xa_inputCond (handle, 2, data_switch->couplage, data_switch->impedance, data_switch->attenuation, data_switch->filtre))!=0)
		analyse_msg_freq("inputCond",err);
	else if ((err = hp5313xa_trigSens (handle,  2, VI_OFF, 50, data_switch->attenuation, VI_ON, 0))!=0)
		analyse_msg_freq("trigSens",err);
	else if (( err = viSetAttribute (handle, VI_ATTR_TMO_VALUE, 2500)) !=0)
		analyse_msg_VI("Timeout",err);
	hp5313xa_close (handle);
	
	// Initialisation du port parallle pour mesurer RF1.
	val_port_para = inp (0x378);
	val_port_para = val_port_para & (~(data_switch->bit_choisi));
	outp (0x378,val_port_para);


	// Mise  jour des donnes de la structure.
	data_switch->identificateur = id_cont;
	data_switch->stop_interruption = RQS; 
	
	return err;
}

/* Fonction de mise en place de l'interruption sur le freq 1 voie.
Renvoit CMPL (0x0100) en cas de succs.*/
int miseenplace_int_freq_switch(PInstrument *frequencemetre)
{
	infos_frsw *data_switch = frequencemetre->structure;
	int err=0;
	char chaine[255];				  
	unsigned short addr_GPIB=0;
	//initialisation de la section de code critique
	//InitializeCriticalSection(&criticalSection);

	//err = ibclr (data_switch->identificateur); 

	err=ibwrt(data_switch->identificateur,"*ESE 1",strlen("*ESE 1"));
	// Activation de l'interruption  la fin des mesures sur le frquencemtre
	if ((err = ibwrt (data_switch->identificateur, "*SRE 32",strlen("*SRE 32"))) & ERR) // Activation
		{
		sprintf(chaine,"Impossible d'initialiser l'interruption\nsur le frquencemtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_switch->identificateur,iberr,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		return err;
		}

	// Nettoyage du registre d'interruptions
	ibecr(data_switch->identificateur,"*ESR?");
	ibrd(data_switch->identificateur,chaine,10);
	
	// Mise en place de l'interruption
	err = ibnotify (data_switch->identificateur, data_switch->stop_interruption, CB_interruption_freq_switch,data_switch);
	if (err & ERR)
		{
		sprintf(chaine,"Impossible d'initialiser le frquencemtre\nd'adresse %d",addr_GPIB);
		MessagePopup("Attention !!!",chaine);
		return -1;
		}
	// lancement d'une mesure
	if ((err = ibecr (data_switch->identificateur, ":INIT:IMM;*WAI;*OPC"))& ERR) // dclenchement immdiat de la mesure
		{
		sprintf(chaine,"Impossible de lancer une mesure\nsur le frquencemtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_switch->identificateur,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		ibnotify (data_switch->identificateur, 0, CB_interruption_freq_switch,data_switch); //Dsactivation de l'interruption
		return err;
		}
	return err;
}
	

int __stdcall CB_interruption_freq_switch(int boardOrDevice, int localsta, int localerr, long localcntl, void *callbackData)
{
	infos_frsw *data_switch;
	int val_port_para=0;
	extern int Lock_port_parallele;

	char chaine[256];
	int err;

	switch (localsta)
	{
		case ERR:
			return 0;
		default:
	
			data_switch = (infos_frsw *)callbackData;
			// Prise du port parallle
			CmtGetLock (Lock_port_parallele);
			if (data_switch->etat_switch == 1)
			{
				val_port_para = inp (0x378);
				val_port_para = val_port_para & (~(data_switch->bit_choisi));
				outp (0x378,val_port_para); // La mesure s'effectue sur RF1
			}
			else
			{
				val_port_para = inp (0x378);
				val_port_para = val_port_para | data_switch->bit_choisi;
				outp (0x378,val_port_para); // La mesure s'effectue sur RF2
			}
			// Prise du port parallle
			CmtReleaseLock (Lock_port_parallele);				
	
			// Enlever le bit RQS
			err = ibrsp(boardOrDevice,chaine); 
			// Demander la frquence au frquencemtre
			err = ibwrt(boardOrDevice,"FETCH:FREQ?",strlen("FETCH:FREQ?"));
			//Lecture de  la chaine
			err = ibrd (boardOrDevice, chaine, 255);
	
			chaine[ThreadIbcntl()]='\0'; //Ajout du caractere de fin de chaine
						  //"+1.2346666666656E+006."
			err = sscanf (chaine, "%lf", &(data_switch->zone_memoire[data_switch->etat_switch])); //transformation en flottant
    
		    // Incrmentation du nombre de mesures.
		    data_switch->numero_mesure[data_switch->etat_switch]++;
      
			if (err==EOF)		
				data_switch->zone_memoire[data_switch->etat_switch]=18.0; // Debug du sscanf -> Si erreur, envoie 18  la place de la mesure.
		
			// Basculement du switch.
			data_switch->etat_switch = (!data_switch->etat_switch);  		
		
			// Nettoyage du registre d'interruptions
			ibecr(boardOrDevice,"*ESR?");
			ibrd(boardOrDevice,chaine,10);

			// Petit dlai pour laisser au temps le temps de laisser  la chaine de switch de basculer
			Delay(0.01); // 10 ms
	
			// Relancement d'une mesure
			ibecr (boardOrDevice, ":INIT:IMM;*OPC"); // dclenchement immdiat
	
			return data_switch->stop_interruption; // Pour ractiver l'interruption
	}
}

static void arreter_interruption_freq_switch(PInstrument *frequencemetre)
{
	infos_frsw *data_switch = frequencemetre->structure;
	
	if (data_switch->stop_interruption == 0) // dj arrt
		return;

	//dsactivation de l'interruption
	data_switch->stop_interruption = 0;
	
	//libration de l'identificateur
	//ibonl (data_switch->identificateur, 0);
}

static void frequencemetre_SW_nouveau(PInstrument **frequencemetre)
{
	*frequencemetre = (PInstrument *)malloc(sizeof(PInstrument));
	memcpy (*frequencemetre, &freq_sw, sizeof(PInstrument));
	(*frequencemetre)->structure = (infos_frsw *)malloc(sizeof(infos_frsw));
	((infos_frsw *)(*frequencemetre)->structure)->adresse_gpib = 3;
	sprintf(((infos_frsw *)(*frequencemetre)->structure)->nom_instru, "Frquencemtre Switch");
}

static void frequencemetre_SW_liberer(PInstrument *instru)
{
	free(instru->structure);
	free(instru);
}

static void frequencemetre_SW_get_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(nom_instru, ((infos_frsw *)instrument->structure)->nom_instru, TAILLE_NOM_INSTRU-1);
}

static void frequencemetre_SW_set_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(((infos_frsw *)instrument->structure)->nom_instru, nom_instru, TAILLE_NOM_INSTRU-1);
	((infos_frsw *)instrument->structure)->nom_instru[TAILLE_NOM_INSTRU-1] = NULL;
}

static void frequencemetre_SW_configure(PInstrument *frequencemetre)
{
	infos_frsw *data_switch = frequencemetre->structure;
	int panel;
	if ((panel = LoadPanel (0, "UIR_frequencemetre_SW.uir", Conf_FrSw)) < 0)
		return ;

	SetCtrlVal (panel, Conf_FrSw_NUM_Conf_Adr_Freq, data_switch->adresse_gpib);
	SetCtrlVal (panel, Conf_FrSw_RING_Conf_Sw_Freq, data_switch->bit_choisi);
	SetCtrlVal (panel, Conf_FrSw_RING_Conf_Digit_Freq, data_switch->digits);
	SetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Impedan, data_switch->impedance);
	SetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Filtre, data_switch->filtre);
	SetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Couplag, data_switch->couplage);
	SetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Attenua, data_switch->attenuation);
						
	SetCtrlAttribute (panel, Conf_FrSw_B_Conf_OK, ATTR_CALLBACK_DATA, data_switch);
	DisplayPanel (panel); // Affiche la fentre "CONFIGURATION DE L'APPAREIL" pour un frquencemtre
}

static void frequencemetre_SW_getconfig(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{
	infos_frsw *data = instrument->structure;

	*p_strct = (void*)data;
	*p_size = sizeof(infos_frsw);
	
	*p_bool = 1;
}

static int frequencemetre_SW_nb_voie(PInstrument *frequencemetre)
{
	infos_frsw *data_switch = frequencemetre->structure;
	return data_switch->nombre_voie;
}

static void frequencemetre_SW_get_nom_voie(PInstrument *frequencemetre, int voie, char *nom_voie)
{
	sprintf(nom_voie, "Voie %i", voie);
}

static double frequencemetre_SW_mesure(PInstrument *frequencemetre, int voie)
{
	infos_frsw *data_freq = frequencemetre->structure;
	return data_freq->zone_memoire[voie-1];
}

static unsigned int frequencemetre_SW_numero(PInstrument *frequencemetre, int voie)
{
	infos_frsw *data_freq = frequencemetre->structure;
	if (voie < 1)
		return 0;
	return data_freq->numero_mesure[voie-1];
}

static int frequencemetre_SW_digits(PInstrument *frequencemetre)
{
	infos_frsw *data_freq = frequencemetre->structure;
	return data_freq->digits;
}

int CVICALLBACK FR_SW_OK (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{

int Valeurs_Configuration_Adresse_GPIB = 0;
int Valeurs_Configuration_Switch = 0;
int Valeurs_Configuration_Digit = 0;
int Valeurs_Configuration_Impedance = 0;
int Valeurs_Configuration_Filtre = 0;
int Valeurs_Configuration_Couplage = 0;
int Valeurs_Configuration_Attenuation = 0;

infos_frsw *structu_inst_sw;

structu_inst_sw = (infos_frsw *)callbackData;

	switch (event)
		{
		case EVENT_COMMIT:
		
			GetCtrlVal (panel, Conf_FrSw_NUM_Conf_Adr_Freq, &Valeurs_Configuration_Adresse_GPIB); // Rcupration de l'adresse GPIB
			GetCtrlVal (panel, Conf_FrSw_RING_Conf_Sw_Freq, &Valeurs_Configuration_Switch); // Rcupration du bit port parallle
			GetCtrlVal (panel, Conf_FrSw_RING_Conf_Digit_Freq, &Valeurs_Configuration_Digit); // Rcupration du nombre de digits
			GetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Impedan, &Valeurs_Configuration_Impedance); // Rcupration de la valeur de l'impdance
			GetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Filtre, &Valeurs_Configuration_Filtre); // Rcupration de l'tat du filtre
			GetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Couplag, &Valeurs_Configuration_Couplage); // Rcupration de la nature du couplage
			GetCtrlVal (panel, Conf_FrSw_BINARYSW_Conf_Attenua, &Valeurs_Configuration_Attenuation); // Rcupration de la valeur de l'attnuation
			
			structu_inst_sw->adresse_gpib = Valeurs_Configuration_Adresse_GPIB;
			structu_inst_sw->bit_choisi = Valeurs_Configuration_Switch;
			structu_inst_sw->digits = Valeurs_Configuration_Digit;
			structu_inst_sw->impedance = Valeurs_Configuration_Impedance;
			structu_inst_sw->filtre = Valeurs_Configuration_Filtre;
			structu_inst_sw->couplage = Valeurs_Configuration_Couplage;
			structu_inst_sw->attenuation = Valeurs_Configuration_Attenuation;
			structu_inst_sw->nombre_voie = 2;
			
			// affichage
			// Transformation_donnees_freq_switch (Transformation,structu_inst_sw);
			// Edit_panelCommande (Transformation);
			// list_inst = Inserer_maillon (list_inst, Type_Appareil, (void *)structu_inst_sw); // Cre un maillon ayant pour type une structure infos
	
			DiscardPanel (panel); // Cache la fentre "CONFIGURATION D'UN APPAREIL" pour un frquencemtre
			
			break;
		}
	return 0;
}
