#include "toolbox.h"
#include "hp34401a.h"
#include <ansi_c.h>
#include <userint.h>
#include <visa.h>
#include <gpib.h>
#include "globales.h"
#include <formatio.h>
#include "multimetre.h"
#include "UIR_multi.h"
#include "plugin.h"

PInstrument multi =
{
	NULL, // Rempli par 'nouveau'

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,

	multimetre_nouveau,
	multimetre_liberer,
	multimetre_configure,
	multimetre_getconfig,
	multimetre_get_nom,
	multimetre_set_nom,
	arreter_interruption_multi,
	miseenplace_int_multi,
	initialisation_multi_int,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,
	NULL,
	
	multimetre_nb_voie,
	multimetre_get_nom_voie,
	multimetre_mesure,
	multimetre_numero,
	
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

int CVICALLBACK MULTI_OK (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{

int Valeurs_Configuration_Adresse_GPIB = 0;
int Valeurs_Configuration_Volt_Ampere = 0;

infos_multi *structu_inst;

structu_inst = (infos_multi *)callbackData;		 

	switch (event)
		{
		case EVENT_COMMIT:
		
			GetCtrlVal(panel, Conf_multi_NUM_Conf_Adr_Freq, &Valeurs_Configuration_Adresse_GPIB); // Rcupration de l'adresse GPIB
			GetCtrlVal(panel, Conf_multi_RINGSLIDE_choix_multi, &Valeurs_Configuration_Volt_Ampere); 
			GetCtrlVal(panel, Conf_multi_RG_DIGITS, &(structu_inst->digits));
			
			structu_inst->adresse_gpib = Valeurs_Configuration_Adresse_GPIB;
			structu_inst->Volt_Ampere = Valeurs_Configuration_Volt_Ampere;
				
			DiscardPanel (panel); // Dtruit la fentre "CONFIGURATION D'UN APPAREIL" pour un frquencemtre
			
			break;
		}
	return 0;
}

int initialisation_multi_int(PInstrument *multimetre)
{
	infos_multi *data_multi = multimetre->structure;
	char chaine[255];
	int err=0,id_cont=0;
	short err_short=0;


	// Initialisation Zone Mmoire.
	data_multi->zone_memoire[0]=NotANumber();

	//Initialisation Numro Mesure.
	data_multi->numero_mesure[0] = 0;

	err = ibfind("GPIB0");
	if (err & ERR)
	{
		MessagePopup ("Problme de carte", "Carte GPIB non dtecte");
		return -1;
	}
	
	// Dtection multimtre
	id_cont = ibdev (0, data_multi->adresse_gpib, NO_SAD, T10s, 1, 1);
	ibclr (id_cont);		// Erreur CVI si branche mais pas allume !?
	ibln (id_cont, data_multi->adresse_gpib, NO_SAD, &err_short); 

	if (err_short == 0) 
		{
		sprintf(chaine,"Il faudrait penser  brancher le multimtre  l'adresse %d !!!",data_multi->adresse_gpib);
		MessagePopup ("PAS BON", chaine);
		return -1;
		}
	
	ibecr(id_cont, "*IDN?");
	ibrd(id_cont, chaine, 250);
	if (strstr (chaine, "HEWLETT-PACKARD,34401") == NULL)
	{
		MessagePopup ("Erreur", "Modle non support par ce module !");
		return -1;
	}
	
	sprintf(chaine,"GPIB::%i", data_multi->adresse_gpib);
	
	hp34401a_init (chaine, VI_ON, VI_ON, &data_multi->handmulti);
	
	hp34401a_conf (data_multi->handmulti, data_multi->Volt_Ampere, VI_ON, 0.0, data_multi->digits);
	
	// Mise  jour des donnes de la structure.
	data_multi->identificateur = id_cont;
	data_multi->stop_interruption = RQS; 
	
	return err;
}

/* Fonction de mise en place de l'interruption sur le freq 1 voie
Renvoit CMPL (0x0100) en cas de succs.*/
int miseenplace_int_multi(PInstrument *multimetre)
{
	infos_multi *data_multi = multimetre->structure;
	int err=0;
	char chaine[255];
	unsigned short addr_GPIB=0;
	
	err=ibwrt(data_multi->identificateur,"*ESE 1",strlen("*ESE 1"));
	// Activation de l'interruption  la fin des mesures sur le multimtre
	if ((err = ibwrt (data_multi->identificateur, "*SRE 32",strlen("*SRE 32"))) & ERR) // Activation
		{
		sprintf(chaine,"Impossible d'initialiser l'interruption\nsur le multimtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_multi->identificateur,iberr,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		return err;
		}

	// Enlever le bit RQS
	err = ibrsp(data_multi->identificateur,chaine); 
	// Nettoyage du registre d'interruptions
	ibecr(data_multi->identificateur,"*ESR?");
	ibrd(data_multi->identificateur,chaine,10);
	
	// Mise en place de l'interruption
	err = ibnotify (data_multi->identificateur, data_multi->stop_interruption, CB_interruption_multi,data_multi);
	if (err & ERR)
	{
		sprintf(chaine,"Impossible d'initialiser le multimtre\nd'adresse %d",addr_GPIB);
		MessagePopup("Attention !!!",chaine);
		return -1;
	}
	// lancement d'une mesure
	if ((err = ibecr(data_multi->identificateur, ":INIT;FETC?;*OPC"))& ERR) // dclenchement immdiat
	{
		sprintf(chaine,"Impossible de lancer une mesure\nsur le multimtre d'identificateur %d\n iberr=%x \n ibsta=%x\nibcntl=%x",data_multi->identificateur,ibsta,ibcntl);
		MessagePopup("Attention !!!",chaine);
		ibnotify (data_multi->identificateur, 0, CB_interruption_multi,data_multi->zone_memoire); //Dsactivation de l'interruption
		return err;
	}
	
	return err;
}
	

int __stdcall CB_interruption_multi(int boardOrDevice, int localsta, int localerr, long localcntl, void *callbackData)
{
	infos_multi *data_multi;
	int err;
	
	char chaine[256];

	switch (localsta)
	{
		case ERR:
			return 0;
		default:
	
			data_multi = (infos_multi *)callbackData;

			// Enlever le bit RQS
			err = ibrsp(boardOrDevice,chaine); 

			// Lire la frquence du frquencemtre
			err = ibrd (boardOrDevice, chaine, 255);
			chaine[ThreadIbcntl()]='\0'; //Ajout du caractere de fin de chaine
			
			err = sscanf (chaine, "%lf", &(data_multi->zone_memoire[0])); //transformation en flottant
	
		    // Incrmentation du nombre de mesures.
		    data_multi->numero_mesure[0]++; 

			// Nettoyage du registre d'interruptions
			ibecr(boardOrDevice,"*ESR?");
			ibrd(boardOrDevice,chaine,10);

			// relancement d'une mesure
			err = ibecr(boardOrDevice,":INIT;FETC?;*OPC");
	
			return data_multi->stop_interruption; // Pour ractiver l'interruption
	}
}




static void arreter_interruption_multi(PInstrument *multimetre)
{
	infos_multi *data_multi = multimetre->structure;
	
	if (data_multi->stop_interruption == 0) // dj arrt
		return;
		
	//dsactivation de l'interruption
	data_multi->stop_interruption = 0;
	
	// Attente de la dernire interruption
	//err = ibwait (data_freq->identificateur, RQS);

	//libration de l'identificateur
	//ibonl (data_freq->identificateur, 0);

	//return err;
}

static void multimetre_nouveau(PInstrument **multimetre)
{
	*multimetre = (PInstrument *)malloc(sizeof(PInstrument));
	memcpy (*multimetre, &multi, sizeof(PInstrument));
	(*multimetre)->structure = malloc(sizeof(infos_multi));
	((infos_multi *)(*multimetre)->structure)->adresse_gpib = 22;
	sprintf(((infos_multi *)(*multimetre)->structure)->nom_instru, "Multimtre");
	((infos_multi *)(*multimetre)->structure)->digits = 0;	
}

static void multimetre_liberer(PInstrument *instru)
{
	free(instru->structure);
	free(instru);
}

static void multimetre_get_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(nom_instru, ((infos_multi *)instrument->structure)->nom_instru, TAILLE_NOM_INSTRU-1);
}

static void multimetre_set_nom(PInstrument *instrument, char *nom_instru)
{
	memcpy(((infos_multi *)instrument->structure)->nom_instru, nom_instru, TAILLE_NOM_INSTRU-1);
	((infos_multi *)instrument->structure)->nom_instru[TAILLE_NOM_INSTRU-1] = NULL;
}

static void multimetre_configure(PInstrument *multimetre)
{
	infos_multi *data_multi = multimetre->structure;
	int panel;
	if ((panel = LoadPanel (0, "UIR_multi.uir", Conf_multi)) < 0)
		return ;
					   
	SetCtrlVal (panel, Conf_multi_NUM_Conf_Adr_Freq, data_multi->adresse_gpib);
	SetCtrlVal (panel, Conf_multi_RINGSLIDE_choix_multi, data_multi->Volt_Ampere);
	SetCtrlVal(panel, Conf_multi_RG_DIGITS, data_multi->digits);
						

	SetCtrlAttribute (panel, Conf_multi_CB_OK_MULTI, ATTR_CALLBACK_DATA, data_multi);
	DisplayPanel (panel); // Affiche la fentre "CONFIGURATION DE L'APPAREIL" pour un frquencemtre
}

static void multimetre_getconfig(PInstrument *instrument, void** p_strct, void** p_member, size_t* p_size, char* p_bool)
{

	infos_multi *data = instrument->structure;
	
	*p_strct = (void*)data;
	*p_size = sizeof(infos_multi);
	
	*p_bool = 1;
}

static int multimetre_nb_voie(PInstrument *multimetre)
{
	return 1;
}

static void multimetre_get_nom_voie(PInstrument *multimetre, int voie, char *nom_voie)
{
	sprintf(nom_voie, "Voie 1");
}

static double multimetre_mesure(PInstrument *multimetre, int voie)
{
	infos_multi *data_multi = multimetre->structure;
	return data_multi->zone_memoire[0];
}

static unsigned int multimetre_numero(PInstrument *multimetre, int voie)
{
	infos_multi *data_multi = multimetre->structure;
	return data_multi->numero_mesure[0];
}

