#ifndef PLUGIN_H
#define PLUGIN_H

#define TAILLE_NOM_VOIE 20
#define TAILLE_NOM_INSTRU 40
#define HISTORIQUE_2D 50

typedef struct ptr
{
	double *mesure;
	double *numero;
	
	// Attention buffer linaires !
	double* points_mid;
	double* points_long;
	int cpt_mid;
	int cpt_long;
	
	int profondeur;
	int ptr_mobile;
	int dernier_indice;
	void *instrument; // Le void* c'est parce que les structures ont besoin de se connaitre l'une l'autre, et je sais pas faire autrement qu'avec un void*
	char nom_voie[TAILLE_NOM_VOIE+TAILLE_NOM_INSTRU];
	int tatoo;
	int voie;
} ptr_buff_circu;

typedef struct ptr2D
{
	double *mesure[100];
	double *indices[100];
	double numero[100];
	int ptr_mobile;
	int dernier_indice;
	void *instrument; // Le void* c'est parce que les structures ont besoin de se connaitre l'une l'autre, et je sais pas faire autrement qu'avec un void*
	char nom_voie[TAILLE_NOM_VOIE+TAILLE_NOM_INSTRU];
	int tatoo;
	int voie;
	int taille_voie;
} ptr_buff2D_circu;

typedef struct PInstrument
{
	void	*structure;
	
	// Fonctions  disposition
	int		(*gestion_lock) (char *lock_requis);
	void	(*init_liste_voie) (int panel, int control_id);
	ptr_buff_circu*		(*get_buffer_from_tatoo) (char *tatoo_voie);
	void	(*init_liste_voie2D) (int panel, int control_id);
	ptr_buff2D_circu*		(*get_buffer2D_from_tatoo) (char *tatoo_voie);

	// Fonctions obligatoirement  fournir
	void	(*nouveau) (struct PInstrument **instrument);
	void	(*liberer) (struct PInstrument *instrument);
	void	(*configure) (struct PInstrument *instrument);
	void	(*get_memory) (struct PInstrument *instrument, void** p_strct, void** p_member, size_t* size, char* bool);
	void	(*get_nom_instru) (struct PInstrument *instrument, char *nom_instru);
	void	(*set_nom_instru) (struct PInstrument *instrument, char *nom_instru);
	
	// Fonction optionnelles
	void	(*stop) (struct PInstrument *instrument);
	int		(*start) (struct PInstrument *instrument);
	int		(*init) (struct PInstrument *instrument);
	int		(*status) (struct PInstrument *instrument);
	void	(*execute) (struct PInstrument *instrument, char *chaine);
	void	(*piloter) (struct PInstrument *instrument); // Pour afficher la fentre de pilotage
	
	// Fonctions de mesures 2D (les 6 ou aucune)
	int		(*nb_voie2D) (struct PInstrument *instrument);
	void	(*get_nom_voie2D) (struct PInstrument *instrument, int voie, char *nom_voie);
	int		(*get_taille_voie2D) (struct PInstrument *instrument, int voie);
	void	(*mesure2D) (struct PInstrument *instrument, int voie, double *mesure);
	void	(*indices2D) (struct PInstrument *instrument, int voie, double *indice);
	unsigned int (*numero2D) (struct PInstrument *instrument, int voie);

	// Fonctions de calcul 1D (les 4 ou aucune)
	int		(*nb_voie_calcul) (struct PInstrument *instrument);
	void	(*get_nom_voie_calcul) (struct PInstrument *instrument, int voie, char *nom_voie);
	double	(*mesure_calcul) (struct PInstrument *instrument, int voie);
	void	(*calculer_1D) (struct PInstrument *instrument); //, ptr_buff_circu *buffer_circu, ptr_buff2D_circu *buffer2D_circu);

	// Fonctions de mesures 1D (les 4 ou aucune)
	int		(*nb_voie_1D) (struct PInstrument *instrument);
	void	(*get_nom_voie_1D) (struct PInstrument *instrument, int voie, char *nom_voie);
	double	(*mesure_1D) (struct PInstrument *instrument, int voie);
	unsigned int (*numero_1D) (struct PInstrument *instrument, int voie);
	
	// Fonctions de calcul 2D (les 5 ou aucune)
	int		(*nb_voie_calcul2D) (struct PInstrument *instrument);
	void	(*get_nom_voie_calcul_2D) (struct PInstrument *instrument, int voie, char *nom_voie);
	int		(*get_taille_voie_calcul2D) (struct PInstrument *instrument, int voie);
	void	(*calcul2D) (struct PInstrument *instrument, int voie, double *mesure);
	void	(*i_calcul2D) (struct PInstrument *instrument, int voie, double *mesure);
	void	(*calculer_2D) (struct PInstrument *instrument); //, ptr_buff_circu *buffer_circu, ptr_buff2D_circu *buffer2D_circu);
} PInstrument;

// Status du pilote
#define STATUS_OQP	0
#define STATUS_PRET	1

#endif

