/*-------------------------------------------
reader.c
Lecture des en-tte des sauvegardes des configs
JD.
07/07/2006
--------------------------------------------*/

#include <userint.h>      
#include <ansi_c.h>

#include "reader.h"

// Initialise la structure
int reader_init(DICTIONARY* dico)
{

	// On remplit tout de zros.
	memset(dico,0,sizeof(DICTIONARY));

	return 1;
}

// Libre la mmoire	
int reader_free(DICTIONARY* dico)
{
	int i;
	
	for(i=0;i<dico->string_N;i++)
	{
		free( (dico->string_values)[i] );
		free( (dico->string_words)[i] );
	}
	
	for(i=0;i<dico->double_N;i++)
		free( (dico->double_words)[i] );
		
	for(i=0;i<dico->int_N;i++)
		free( (dico->int_words)[i] );
		
	if(dico->string_values) free(dico->string_values);
	if(dico->string_words) free(dico->string_words);
	
	if(dico->double_words) free(dico->double_words);
	if(dico->double_values) free(dico->double_values);
	
	if(dico->int_words) free(dico->int_words);
	if(dico->int_values) free(dico->int_values);
	
	return 1;
}

// Rajoute la string pass en param  la liste.
int reader_add_string_to_list(char* string, char*** p_list,int N)
{

	if( !(*p_list) ) // mmoire non allou pour l'instant
	{
		*p_list = (char**)malloc(sizeof(char*));
		
		if( !(*p_list) ) return 0;
		
		**p_list = (char*)malloc( (strlen(string)+1)*sizeof(char) );
		
		strcpy(**p_list,string);
		
		return 1;
	}	
	
	*p_list = (char**)realloc(*p_list,N*sizeof(char*));
	
	if( !(*p_list) ) return 0;
	
	(*p_list)[N - 1] = (char*)malloc( (strlen(string)+1)*sizeof(char) );
	
	if( !(*p_list)[N -1] ) return 0;
	
	strcpy( (*p_list)[N - 1],string);
				  
	return 1;
}

// Rajoute l'int pass en param  la liste.
int reader_add_int_to_list(int value, int** p_list, int N)
{
	if(!(*p_list)) // mmoire non allou pour l'instant
	{
		*p_list = (int*)malloc(sizeof(int));
		
		if(!(*p_list)) return 0;
		
		**p_list = value;
				
		return 1;
	}	
	
	*p_list = (int*)realloc(*p_list,N*sizeof(int));
	
	if(!(*p_list)) return 0;
	
	(*p_list)[N - 1] = value;
				  
	return 1;

}

// Rajoute le double pass en param  la liste.
int reader_add_double_to_list(double value, double** p_list, int N)
{
	if(!*p_list) // mmoire non allou pour l'instant
	{

		*p_list = (double*)malloc(sizeof(double));
		
		if( !(*p_list) ) return 0;
		
		**p_list = value;
				
		return 1;
	}	
	
	*p_list = (double*)realloc(*p_list,N*sizeof(double));
	
	if(!*p_list) return 0;
	
	(*p_list)[N - 1] = value;
				  
	return 1;

}

// Lit un fichier, le parse et stocke tout ce qui va bien.
int reader_load_file( char* filename, DICTIONARY* dico)
{

	char line[256];
	
	char lvalue[256];
	char rvalue[256];
	
	char temp[256];
	
	int i_tmp;
	double d_tmp;
	
	int type;

	FILE* f = fopen(filename,"r");
	
	while( fgets(line,256,f) )
	{
		*rvalue = *lvalue = *temp = '\0';
	
		// Supprime les espaces et les commentaires.
		reader_extract_string(line);	
		
		// Regardons si il reste quelque chose  se
		// mettre sous la dent.
		if( strlen(line) == 0 ) continue;
		
		// On extrait lvalue et rvalue.
		if( !reader_extract_values(line, lvalue, rvalue) ) continue;
		
		// On recherche le type de la rvalue.
		type = reader_return_rvalue_type(rvalue);
		
		switch(type)
		{
			case 1:  // int
			
				reader_convert_rvalue_to_int(rvalue,&i_tmp);
				
				(dico->int_N)++; // Un nouvel lment dans la liste !
				reader_add_string_to_list( lvalue, &(dico->int_words), dico->int_N );
				reader_add_int_to_list( i_tmp, &(dico->int_values), dico->int_N);
			
				break;
			
			case 2:  // double
			
				reader_convert_rvalue_to_double(rvalue,&d_tmp);

				(dico->double_N)++; // Un nouvel lment dans la liste !
				reader_add_string_to_list( lvalue, &(dico->double_words), dico->double_N );
				reader_add_double_to_list( d_tmp, &(dico->double_values), dico->double_N);
			
				break;
			
			case 3: // string
			
				reader_convert_rvalue_to_string(rvalue,temp);

				(dico->string_N)++; // Un nouvel lment dans la liste !
				reader_add_string_to_list( lvalue, &(dico->string_words), dico->string_N );
				reader_add_string_to_list( temp, &(dico->string_values), dico->string_N);
			
				break;
			
			default: break;
		}
		
	}
	
	fclose(f);

	return 1;
}

// Retourne l'entier associ au mot pass en paramtre
int reader_return_int( char* word, DICTIONARY* dico)
{

	int i;

	for(i=0;i<(dico->int_N);i++)
		if( !strcmp(word,(dico->int_words)[i]) )
			return (dico->int_values)[i]; // Got it !
	

	return 0;
}

// Retourne le double associ au mot pass en paramtre
double reader_return_double( char* word, DICTIONARY* dico)
{
  	int i;

	for(i=0;i<(dico->double_N);i++)
		if( !strcmp(word,(dico->double_words)[i]) )
			return (dico->double_values)[i]; // Got it !
	

	return 0.0;
}

// Retourne la chaine associ au mot pass en paramtre
char* reader_return_string( char* word, DICTIONARY* dico)
{
  	int i;

	for(i=0;i<(dico->string_N);i++)
		if( !strcmp(word,(dico->string_words)[i]) )
			return (dico->string_values)[i]; // Got it !
	

	return NULL;
}

// Extrait la partie utile d'une chaine
int reader_extract_string( char* string )
{
	int i,j;
	char* temp;
	char bool = 0;
	
	int N = strlen(string);

	if(!string) return 0;
	
	temp = (char*)malloc( (N+1)*sizeof(char) );

	if(!temp) return 0;
	
	*temp = '\0';
	
	for(i=0,j=0;i<N;i++)
	{
		if( string[i] == '"' )
			bool = !bool;
		
		if( !bool && string[i] == ' ' ) continue;
		
		if( string[i] == ';' ) break;
		
		temp[j] = string[i];
		
		temp[j+1] = '\0';
		
		j++;
	}

	strcpy(string,temp);
	
	free(temp);
	
	return 1;
}

// Extrait la valeur de gauche et celle de droite
int reader_extract_values( char* in, char* lvalue, char* rvalue )
{
	int i,j;
	char loutre = 0;
	int N = strlen(in);
	
	for(i=0,j=0;i<N;i++)
	{
		if( in[i] == '=' )
		{
			loutre = 1;
			break;
		}
		
		lvalue[j] = in[i];
		
		lvalue[j+1] = '\0';
		
		j++;
	}
	
	if(!loutre)   // Pas de loutre, pas de chocolat.
		return 0;

	for(j=0;i<N;i++)
	{
		if( in[i] == '=' ) continue;
		
		rvalue[j] = in[i];
		
		rvalue[j+1] = '\0';
		
		j++;
	}


	return 1;
}

// Retourne le type de la rvalue.
// 1-int 2-double 3-string
int reader_return_rvalue_type( char* rvalue )
{

	// NDJD:
	// Cette fonction ne vrifie en aucun
	// cas que la rvalue est correcte.
	// En effet elle prendra er.12
	// pour un double.
	// Ce n'est pas un drame puisque les fonctions
	// de convertions str->int s'en chargeront.

	int N = strlen(rvalue);
	
	if( (rvalue[0] == '"') && (rvalue[N-1] == '"') )
		return 3; // String  n'en pas douter :)
	
	if( strchr(rvalue,'.') )
		return 2; // Pas une string donc un double
	
	return 1; // Donc un int
}

// Convertit la rvalue vers une string.
int reader_convert_rvalue_to_string( char* rvalue, char* string )
{
	int N = strlen(rvalue);

	if( N<3 ) return 0;
	
	// Petit bourrinage pour extraire le xxxx dans "xxxx"
	memcpy( string, rvalue + 1, (N-2)*sizeof(char) );
	
	string[N-2]='\0';

	return 1;
}

// Convertit la rvalue vers un int.
int reader_convert_rvalue_to_int( char* rvalue, int* value )
{
	
	*value = (int)strtol(rvalue, (char**)NULL, 10);
	
	return 1;
}

// Convertit la rvalue vers un double.
int reader_convert_rvalue_to_double( char* rvalue, double* value )
{

	*value = strtod(rvalue, (char**)NULL);

	return 1;
}

